# -*- coding: utf-8 -*-
import typing

# Import specific members from typing used in hints
from typing import (
    Any,
    Callable,
    Dict,
    FrozenSet,
    Generic,
    Iterable,
    Iterator,
    List,
    Mapping,
    Optional,
    Sequence,
    Set,
    Tuple,
    TypeVar,
    Union,
    overload,
)

import datetime
from enum import Enum

from mh_operator.SDK import Agilent, System

# Stubs for namespace: CompositionDB

class CompositionsStorage:  # Class
    def __init__(self) -> None: ...
    @staticmethod
    def GetMasses(neutralAndNaturalOnly: bool) -> List[float]: ...
    @staticmethod
    def Open() -> None: ...
    @staticmethod
    def FormulaExists(formula: str) -> bool: ...
    @staticmethod
    def GetFormulaCount(neutralAndNaturalOnly: bool) -> int: ...
    @staticmethod
    def Close() -> None: ...
    @staticmethod
    def GetIsomerAndTautomerInfo(
        formula: str, isomerCount: int, tautomerGroupCount: int
    ) -> None: ...
    @staticmethod
    def GetFormulaMasses(neutralAndNaturalOnly: bool) -> Dict[str, float]: ...
    @staticmethod
    def InsertFormula(
        formula: str, mass: float, isomerCount: int, tautomerGroupCount: int
    ) -> None: ...



# -*- coding: utf-8 -*-
import typing

# Import specific members from typing used in hints
from typing import (
    Any,
    Callable,
    Dict,
    FrozenSet,
    Generic,
    Iterable,
    Iterator,
    List,
    Mapping,
    Optional,
    Sequence,
    Set,
    Tuple,
    TypeVar,
    Union,
    overload,
)

import datetime
from enum import Enum

from mh_operator.SDK import Agilent, System

from .Agilent.MassSpectrometry.DataAnalysis import (
    CeCompositeSpectraMakingScheme,
    IFunction,
    IIonSpeciesDetails,
    IIsotopeClusterBase,
    IModificationSpecies,
    IModificationUnit,
    IsolationWindowType,
    SpectralPeak,
)
from .Agilent.MassSpectrometry.DataAnalysis.MassHunter import (
    CompoundFilterParameters,
    IChargeState,
    ICoelutionGroup,
    ICompound,
    IIsotope,
    IIsotopeCluster,
    IProtein,
    IRawSpectrumAccessor,
    ISignal3D,
    IsotopeCharacteristics,
    SpectrumSetMetadata,
)
from .Biochemistry import CompositionModel
from .CompoundFilters import IFilterable, IFilterableMfe
from .Definitions import IMass_TimeSeparatedObject
from .Mathematics import ClusteringSequential, RangeDouble, RangeInt, Regression, Vector
from .Mfe import IsotopeCharacter

# Stubs for namespace: BasicTypes

class AlternativeIonSpecies:  # Class
    @staticmethod
    def Parse(
        expression: str,
        multimerNumber: int,
        adducts: str,
        isProtonated: bool,
        neutralModification: str,
    ) -> None: ...

class CXY(BasicTypes.OrderedXY):  # Class
    def __init__(self) -> None: ...

class CalibReader:  # Class
    def __init__(self) -> None: ...
    @staticmethod
    def ReadCalib(spectrumAccessor: IRawSpectrumAccessor) -> BasicTypes.TOFCalib: ...

class ChromMetadata:  # Class
    @overload
    def __init__(self) -> None: ...
    @overload
    def __init__(
        self,
        ionization: BasicTypes.Ionization,
        instrumentInfo: BasicTypes.ChromMetadata.InstrumentInfo,
    ) -> None: ...
    @overload
    def __init__(
        self, accessor: IRawSpectrumAccessor, highResolutino: bool
    ) -> None: ...

    AcqRTRange: RangeDouble
    AcqSpectralXRange: RangeDouble
    CalibratedTimeRangeInNS: RangeDouble
    CurrentRTRange: RangeDouble  # readonly
    CurrentScanCount: int  # readonly
    CurrentSpectralXRange: RangeDouble
    FullSaturationLevel: float
    Ionization: BasicTypes.Ionization
    IsotopesUnresovled: bool
    MSLevel: int
    MZErrorFunction: BasicTypes.ChromMetadata.MZError  # readonly
    MZPeakWidthFunction: IFunction  # readonly
    ProfileSpectraAvailable: bool
    RTPeakWidth: float
    ScanRate: float  # readonly
    ScanTimes: List[float]
    ScanWidths: List[float]  # readonly
    TheInstrumentInfo: BasicTypes.ChromMetadata.InstrumentInfo

    def GetNearestScanIndex(self, rt: float) -> int: ...
    def GetScanRange(self, rtRange: RangeDouble) -> RangeInt: ...
    def TruncateScanRange(self, scanRange: RangeInt) -> None: ...
    def Clone(self) -> BasicTypes.ChromMetadata: ...

    # Nested Types

    class InstrumentInfo:  # Class
        def __init__(self, type: SpectrumSetMetadata.MSInstrumentType) -> None: ...

        Type: SpectrumSetMetadata.MSInstrumentType
        Version: str

    class MZError:  # Class
        def __init__(
            self, coeffHighSNRCase: List[float], coeffLowSNRCase: List[float]
        ) -> None: ...
        def LowSNRCase(self, mz: float, scalingFactor: float) -> float: ...
        def HighSNRCoeff(self, scalingFactor: float) -> List[float]: ...
        def Clone(self) -> BasicTypes.ChromMetadata.MZError: ...
        def LowSNRCoeff(self, scalingFactor: float) -> List[float]: ...
        def HighSNRCase(self, mz: float, scalingFactor: float) -> float: ...

class Chromatogram(BasicTypes.CXY):  # Class
    @overload
    def __init__(self, rt: List[float]) -> None: ...
    @overload
    def __init__(self, rt: List[float], intensities: List[float]) -> None: ...
    def __getitem__(self, i: int) -> float: ...
    def __setitem__(self, i: int, value_: float) -> None: ...
    Length: int  # readonly
    RTRange: RangeDouble  # readonly
    ScanTimes: List[float]  # readonly

    def RT2Scan(self, rt: float) -> float: ...
    def ApplyTimeShiftCorrection(
        self, shiftModel: Regression, factor: float
    ) -> None: ...
    @staticmethod
    def NewCalculatePeakWidth(
        centroid: float, data: List[BasicTypes.Signal3D]
    ) -> float: ...
    def GetX(self, index: int) -> float: ...
    def GetY(self, index: int) -> float: ...
    def GetNearestScan(self, rt: float) -> int: ...
    @staticmethod
    def CalculatePeakWidth(
        centroid: float, data: List[BasicTypes.Signal3D]
    ) -> float: ...

class CoelutionGroup(ICoelutionGroup):  # Class
    @overload
    def __init__(self, compounds: List[Any]) -> None: ...
    @overload
    def __init__(self) -> None: ...

    AllPeaks: System.Collections.Generic.List[BasicTypes.Peak]  # readonly
    Compounds: List[ICompound]  # readonly
    ID: int
    MaxVolume: float  # readonly
    RT: float  # readonly
    RetentionTime: float  # readonly
    Saturated: bool  # readonly

    def Split(
        self, maxRTSpread: float, penalityCompoundCount: float
    ) -> System.Collections.Generic.List[BasicTypes.CoelutionGroup]: ...
    @staticmethod
    def GetCoelutionGroups(
        compounds: List[BasicTypes.MfeCompound],
        filterParameters: CompoundFilterParameters,
        maxGroupSpread: float,
    ) -> List[BasicTypes.CoelutionGroup]: ...
    def SortCompoundsByAbundance(self) -> None: ...

class CompoundInfo(BasicTypes.ICompoundInfo):  # Class
    def __init__(
        self, compoundName: str, formula: str, molfile: str, retentionTime: float
    ) -> None: ...

    CompoundName: str  # readonly
    Formula: str  # readonly
    IonSource: SpectrumSetMetadata.MSSourceType  # readonly
    Molfile: str  # readonly
    RetentionTime: float  # readonly

class EICCollection:  # Class
    @overload
    def __init__(self) -> None: ...
    @overload
    def __init__(
        self,
        template: BasicTypes.EICCollection,
        signals: System.Collections.Generic.List[BasicTypes.Signal3D],
    ) -> None: ...
    @overload
    def __init__(
        self,
        resolutionBinStarts: List[float],
        signalBinBoundaries: List[float],
        signals: System.Collections.Generic.List[BasicTypes.Signal3D],
    ) -> None: ...
    @overload
    def __init__(
        self,
        resolutionBinStarts: List[float],
        mzSortedBins: List[BasicTypes.EICCollection.SignalBin],
    ) -> None: ...

    BinCount: int  # readonly
    IsBinned: bool  # readonly
    ResolutionBinStarts: List[float]  # readonly
    SignalBinBoundaries: List[float]  # readonly
    SignalBins: List[BasicTypes.EICCollection.SignalBin]  # readonly

    def FindResolutionBinRange(self, spectralXRange: RangeDouble) -> RangeInt: ...
    def RoundupRange(
        self, mzRange: RangeDouble, binChoice: BasicTypes.XYCollection.EICBinChoice
    ) -> RangeDouble: ...
    def SetBinningInfo(
        self,
        resulotionBinStarts: List[float],
        signalBins: List[BasicTypes.EICCollection.SignalBin],
    ) -> None: ...
    def GetResolutionBinUpperBoundary(self, mz: float) -> float: ...
    @overload
    def GetMZRange(self, binIndex: int) -> RangeDouble: ...
    @overload
    def GetMZRange(self, binRange: RangeInt) -> RangeDouble: ...
    def GetNoiseLevel(self, bin: int, scan: int) -> float: ...
    def GetBinRange(self, mz: float) -> RangeDouble: ...
    def SetNoiseLevels(
        self, noiseLevels: System.Collections.Generic.List[Vector]
    ) -> None: ...
    def GetBinCenter(self, bin: int) -> float: ...
    def FindBin(self, mz: float) -> int: ...
    def ClearSegment(self, binRange: RangeInt) -> None: ...
    def FindResolutionBin(self, mz: float) -> int: ...

    # Nested Types

    class SignalBin:  # Class
        def __init__(self) -> None: ...

        Centroid: float
        NoiseLevel: Vector
        NonZeroSignals: System.Collections.Generic.List[BasicTypes.Signal3D]
        StartingLoc: float
        StorageOffset: int

        SignalCount: int

        def SetSignalInfo(
            self, storageOffset: int, qualifiedSignalCount: int
        ) -> None: ...
        def EIC(self, scanCount: int) -> List[float]: ...

class FragmentDataInfo:  # Class
    def __init__(self) -> None: ...

    CeCompositeScheme: CeCompositeSpectraMakingScheme
    FragmentInfo: BasicTypes.FragmentSpectraInfo
    UseFormulasInFiles: bool

    def Clone(self) -> BasicTypes.FragmentDataInfo: ...
    def BinaryWrite(self, writer: System.IO.BinaryWriter) -> None: ...
    def BinaryRead(self, reader: System.IO.BinaryReader) -> None: ...

class FragmentIon:  # Class
    def __init__(
        self,
        isotopePeaks: List[SpectralPeak],
        z: int,
        trueFrgament: bool,
        hasIsotopeData: bool,
        isotopePatternIntact: bool,
    ) -> None: ...

    HasIsotopeData: bool  # readonly
    Height: float  # readonly
    Intensity: float  # readonly
    IonMass: float  # readonly
    IsotopePatternIntact: bool  # readonly
    NeutralizedMass: float  # readonly
    SpectralPeaks: List[SpectralPeak]  # readonly
    TrueFragment: bool  # readonly
    Z: int  # readonly

class FragmentSpectraInfo:  # Class
    def __init__(self) -> None: ...

    FragmentHeightAccuracy: float  # static
    FragmentMassAccuracy: List[float]  # static
    HeightAccuracy: float
    IsolationWindow: IsolationWindowType
    IsotopeCharacteristics: IsotopeCharacteristics
    MassAccurancyCoefficients: List[float]
    MaxFragmentIonCount: int
    MaxIonMass: float
    MinSNRatio: float
    ParentIsPortonated: bool
    TreatHeavestIonAsParent: bool

    def Clone(self) -> BasicTypes.FragmentSpectraInfo: ...
    def BinaryWrite(self, writer: System.IO.BinaryWriter) -> None: ...
    def BinaryRead(self, reader: System.IO.BinaryReader) -> None: ...

class GenericCompound(IFilterable, IMass_TimeSeparatedObject):  # Class
    def __init__(self, time: float, mass: float, abundance: float) -> None: ...

    Abundance: float  # readonly
    Height: float  # readonly
    ID: str  # readonly
    Mass: float  # readonly
    RetentionTime: float  # readonly
    SeparationTime: float  # readonly

class IChromatographyObject(object):  # Interface
    RT: float  # readonly

class ICompoundInfo(object):  # Interface
    CompoundName: str  # readonly
    Formula: str  # readonly
    IonSource: SpectrumSetMetadata.MSSourceType  # readonly
    Molfile: str  # readonly
    RetentionTime: float  # readonly

class ICompoundPrecursorProductDataSet(object):  # Interface
    CompoundInfo: BasicTypes.ICompoundInfo  # readonly
    PrecursorDataSets: Sequence[BasicTypes.IPrecursorDataSet]  # readonly

    def GetTotalProductSpectraCount(self) -> int: ...

class IIntensity2D(BasicTypes.ILocation2D):  # Interface
    Intensity: float  # readonly

class ILocation2D(object):  # Interface
    XLocation: float  # readonly
    YLocation: float  # readonly

class IPrecursorDataSet(object):  # Interface
    EIData: bool  # readonly
    IonSpecies: IModificationSpecies  # readonly
    IsolationWindow: IsolationWindowType  # readonly
    PrecursorFullSaturation: float  # readonly
    PrecursorIsotopePeaks: List[SpectralPeak]  # readonly
    PrecursorMz: float  # readonly

    def GetPrecursorZ(self, zValueNeedsConfirm: bool) -> int: ...
    def GetCollisionEnergies(self) -> Sequence[float]: ...
    def GetProductSpectraFullSaturation(self, collisionEnergy: float) -> float: ...
    def GetProductSpectrum(self, collisionEnergy: float) -> List[SpectralPeak]: ...
    def GetProductCompositeSpectrum(
        self,
        scheme: CeCompositeSpectraMakingScheme,
        productMassErrorCoefficients: List[float],
    ) -> List[SpectralPeak]: ...
    def ConfirmZValue(self) -> None: ...

class IRTShiftable(BasicTypes.IIntensity2D, BasicTypes.ILocation2D):  # Interface
    def ShiftRT(self, shiftModel: Regression, factor: float) -> None: ...

class ISpectrum(object):  # Interface
    Calib: BasicTypes.TOFCalib
    RT: float
    SignalCount: int  # readonly
    SummedHeight: float  # readonly

    def CalculateSummmedHeight(
        self, XRange: RangeDouble, searchStartSignalIndex: int
    ) -> float: ...
    def GetImage(self, binRange: RangeInt) -> List[float]: ...

class IonSpeciesAdapter(IIonSpeciesDetails):  # Class
    def __init__(self, species: IModificationSpecies) -> None: ...

    BaseCount: int
    ElectronCount: int
    ModifierCount: int
    ModifierFormula: str
    NeutralLoss: str
    ShorthandSpeciesFormula: str  # readonly

class Ionization:  # Class
    @overload
    def __init__(
        self,
        type: SpectrumSetMetadata.MSSourceType,
        polarity: SpectrumSetMetadata.MSPolarity,
    ) -> None: ...
    @overload
    def __init__(self) -> None: ...

    BaseChargeUnit: IModificationUnit  # readonly
    ThePolarity: SpectrumSetMetadata.MSPolarity
    TheSourceType: SpectrumSetMetadata.MSSourceType

class IonizationChargeType(
    System.IConvertible, System.IComparable, System.IFormattable
):  # Struct
    Electron: BasicTypes.IonizationChargeType = ...  # static # readonly
    Proton: BasicTypes.IonizationChargeType = ...  # static # readonly
    Unkonwn: BasicTypes.IonizationChargeType = ...  # static # readonly

class IsotopeCluster(IIsotopeClusterBase, IIsotopeCluster):  # Class
    @overload
    def __init__(
        self,
        peaks: BasicTypes.PeakList,
        lowestIsotopeMZ: float,
        ionSpecies: IModificationSpecies,
        rtWidth: float,
        isotopeCharacter: IsotopeCharacter,
    ) -> None: ...
    @overload
    def __init__(
        self,
        peaks: BasicTypes.PeakList,
        missingLeadingPeakCount: int,
        lowestIsotopeMZ: float,
        ionSpecies: IModificationSpecies,
        rtWidth: float,
        isotopeCharacter: IsotopeCharacter,
        qScore: float,
    ) -> None: ...
    @overload
    def __init__(
        self, chargeCount: int, isotopeCharacter: IsotopeCharacter
    ) -> None: ...
    @overload
    def __init__(
        self, p: BasicTypes.Peak, chargeCount: int, isotopeCharacter: IsotopeCharacter
    ) -> None: ...
    @overload
    def __init__(
        self,
        pks: BasicTypes.PeakList,
        chargeCount: int,
        isotopeCharacter: IsotopeCharacter,
    ) -> None: ...
    @overload
    def __init__(self, cluster: BasicTypes.IsotopeCluster) -> None: ...
    @overload
    def __init__(self, cluster: BasicTypes.IsotopeCluster, scale: float) -> None: ...
    @overload
    def __init__(
        self, cluster: BasicTypes.IsotopeCluster, isotopeCharacter: IsotopeCharacter
    ) -> None: ...
    @overload
    def __init__(
        self, pks: BasicTypes.PeakList, ionSpeciesExpression: str, z: int
    ) -> None: ...

    Abundance: float  # readonly
    AverageMZ: float  # readonly
    AverageMass: float  # readonly
    ChargeCount: int
    ContiguousPeakLength: int  # readonly
    IonSpecies: IModificationSpecies
    IonSpeciesDetails: IIonSpeciesDetails  # readonly
    IonSpeciesExpression: str  # readonly
    IsProtonated: bool  # readonly
    IsotopeCharacter: IsotopeCharacter  # readonly
    Isotopes: List[IIsotope]  # readonly
    Length: int  # readonly
    LowestIsotopeMZ: float  # readonly
    MassSpread: float  # readonly
    MaxEicSNRatio: float  # readonly
    MaxHeight: float  # readonly
    MissingLeadingPeakCount: int
    MonomerLowestIsotopeMass: float  # readonly
    MultimerNumber: int  # readonly
    NeutralModifications: List[IModificationUnit]  # readonly
    Peaks: BasicTypes.PeakList  # readonly
    PredictedM0FromEachIon: List[float]  # readonly
    QScore: float
    QualityScore: float
    RT: float  # readonly
    RTDeviation: float  # readonly
    RTWidth: float  # readonly
    RetentionTime: float  # readonly
    RetentionTimePeakWidth: float  # readonly
    Saturated: bool  # readonly
    SaturationCorrectedIsotopePattern: List[SpectralPeak]  # readonly
    SpectralPeaks: List[SpectralPeak]  # readonly
    SummedIntensity: float  # readonly
    TotalVolumeExcludingSaturatePart: float  # readonly
    Volume: float  # readonly
    Z: int  # readonly

    def DropInfoLevelTo(self, infoLevel: BasicTypes.MfeCompound.InfoLevel) -> None: ...
    def RedoStatistics(self) -> None: ...
    @staticmethod
    def Merge(
        clusterA: BasicTypes.IsotopeCluster, clusterB: BasicTypes.IsotopeCluster
    ) -> BasicTypes.IsotopeCluster: ...
    @staticmethod
    def ConstructGapPeaks(
        anchor1: BasicTypes.Peak,
        anchor2: BasicTypes.Peak,
        gap: int,
        templete: BasicTypes.Peak,
    ) -> List[BasicTypes.Peak]: ...
    def CalculateRTTruncationBias(self) -> float: ...
    def IsPeakSaturated(self, peakIndex: int) -> bool: ...
    @staticmethod
    def SimplyMerge(
        cluster1: BasicTypes.IsotopeCluster, cluster2: BasicTypes.IsotopeCluster
    ) -> BasicTypes.IsotopeCluster: ...
    def GetSaturationCorrectedIsotopePattern(self) -> BasicTypes.PeakList: ...
    def TrimExtraLeadingPeakIfNeccessary(self) -> None: ...
    @staticmethod
    def EstimateMissingLeadingPeakCount(
        model: CompositionModel,
        mass: float,
        clusters: Sequence[Dict[int, BasicTypes.Peak]],
    ) -> int: ...

    # Nested Types

    class Comparer(
        System.Collections.Generic.IComparer[BasicTypes.IsotopeCluster]
    ):  # Class
        def __init__(self) -> None: ...
        def Compare(
            self, c1: BasicTypes.IsotopeCluster, c2: BasicTypes.IsotopeCluster
        ) -> int: ...

class MSProfile(BasicTypes.CXY, BasicTypes.ISpectrum):  # Class
    @overload
    def __init__(self) -> None: ...
    @overload
    def __init__(
        self,
        spectrumAccessor: IRawSpectrumAccessor,
        scanIndex: int,
        mzRange: RangeDouble,
    ) -> None: ...
    @overload
    def __init__(self, spectrumAccessor: IRawSpectrumAccessor, rt: float) -> None: ...
    @overload
    def __init__(
        self, spectrumAccessor: IRawSpectrumAccessor, rt: float, mzRange: RangeDouble
    ) -> None: ...
    @overload
    def __init__(
        self,
        spectrumAccessor: IRawSpectrumAccessor,
        rtRange: RangeDouble,
        mzRange: RangeDouble,
        requiredType: BasicTypes.XYCollection.SpectrumType,
    ) -> None: ...
    @overload
    def __init__(
        self,
        spectrumAccessor: IRawSpectrumAccessor,
        centerScanIndex: int,
        taper: List[float],
        taperStartIndex: int,
        mzRange: RangeDouble,
    ) -> None: ...
    @overload
    def __init__(
        self, intensities: List[float], x0: float, deltaT: float, rt: float
    ) -> None: ...

    Calib: BasicTypes.TOFCalib
    DataArray: List[float]  # readonly
    FullSaturationLevel: float  # readonly
    Length: int  # readonly
    Peaks: BasicTypes.PeakList
    RT: float
    SignalCount: int  # readonly
    SummedHeight: float  # readonly
    X0: float

    def GetIndex(self, mz: float) -> int: ...
    @staticmethod
    def SpectraCompatible(
        s1: BasicTypes.MSProfile, s2: BasicTypes.MSProfile
    ) -> bool: ...
    def GetImage(self, binRange: RangeInt) -> List[float]: ...
    def GetX(self, index: int) -> float: ...
    def GetY(self, index: int) -> float: ...
    def CreatePeakWidthProportionalModelInIndex(
        self, widthInMZ: IFunction, scalingFactor: float
    ) -> IFunction: ...
    def Clone(self, intensities: List[float]) -> BasicTypes.MSProfile: ...
    @staticmethod
    def CreateStackedSpectrum(
        spectrumAccessor: IRawSpectrumAccessor,
        rtRange: RangeDouble,
        mzRange: RangeDouble,
        doAveraging: bool,
    ) -> BasicTypes.MSProfile: ...
    def CalculateSummmedHeight(
        self, xRange: RangeDouble, searchStartSignalIndex: int
    ) -> float: ...

class MSProfileCompressed(BasicTypes.ISpectrum, BasicTypes.MSProfile):  # Class
    def __init__(self) -> None: ...

    HighSignals: List[int]
    Length: int  # readonly
    LowSignals: List[int]
    MediumSignals: List[int]

    def SetData(self, rawData: List[int]) -> int: ...
    def GetData(self, requiredLength: int) -> List[int]: ...
    def GetImage(self, binRange: RangeInt) -> List[float]: ...

class MfeCompound(
    BasicTypes.IChromatographyObject,
    ICompound,
    IMass_TimeSeparatedObject,
    IFilterableMfe,
    IFilterable,
):  # Class
    @overload
    def __init__(self, main: BasicTypes.IsotopeCluster) -> None: ...
    @overload
    def __init__(self) -> None: ...

    Abundance: float  # readonly
    AllPeaks: BasicTypes.PeakList  # readonly
    AverageMass: float  # readonly
    AverageMassStandardDeviation: float  # readonly
    Clusters: System.Collections.Generic.List[BasicTypes.IsotopeCluster]
    HasOnlySingleIon: bool  # readonly
    Height: float  # readonly
    Intensity: float  # readonly
    IsotopeCharacter: IsotopeCharacter  # readonly
    IsotopeClusters: List[IIsotopeCluster]  # readonly
    LightestCluster: BasicTypes.IsotopeCluster  # readonly
    LowestIsotopeMass: float
    Mass: float  # readonly
    MassSpread: float  # readonly
    MassStandardDeviation: float  # readonly
    MaxHeight: float  # readonly
    MaxVolume: float  # readonly
    MostReliableCluster: BasicTypes.IsotopeCluster
    MostReliableIsotopeCluster: IIsotopeCluster  # readonly
    QScore: float
    QualityScore: float
    RT: float
    RTDeviation: float  # readonly
    RTWidth: float
    RetentionTime: float  # readonly
    RetentionTimePeakWidth: float  # readonly
    Saturated: bool  # readonly
    SeparationTime: float  # readonly
    Volume: float
    XLocation: float  # readonly
    YLocation: float  # readonly
    ZRange: RangeInt
    ZStateCount: int  # readonly
    ZStates: List[int]  # readonly

    def DropInfoLevelTo(self, infoLevel: BasicTypes.MfeCompound.InfoLevel) -> None: ...
    def RedoStatistics(self) -> None: ...
    def Add(self, cluster: BasicTypes.IsotopeCluster) -> None: ...
    @overload
    @staticmethod
    def SortListBy(
        compounds: List[Any],
        sortingType: BasicTypes.MfeCompound.SortingType,
        ascending: bool,
    ) -> None: ...
    @overload
    @staticmethod
    def SortListBy(
        compounds: List[BasicTypes.MfeCompound],
        sortingType: BasicTypes.MfeCompound.SortingType,
        ascending: bool,
    ) -> None: ...
    def SetIsotopeClusters(
        self, clusters: System.Collections.Generic.List[BasicTypes.IsotopeCluster]
    ) -> None: ...

    # Nested Types

    class Comparer(
        System.Collections.Generic.IComparer[ICompound],
        System.Collections.IComparer,
        System.Collections.Generic.IComparer[BasicTypes.MfeCompound],
    ):  # Class
        def __init__(
            self, type: BasicTypes.MfeCompound.SortingType, ascending: bool
        ) -> None: ...
        @overload
        def Compare(self, o1: Any, o2: Any) -> int: ...
        @overload
        def Compare(
            self, c1: BasicTypes.MfeCompound, c2: BasicTypes.MfeCompound
        ) -> int: ...
        @overload
        def Compare(self, c1: ICompound, c2: ICompound) -> int: ...

    class InfoLevel(
        System.IConvertible, System.IComparable, System.IFormattable
    ):  # Struct
        MostDetailed: BasicTypes.MfeCompound.InfoLevel = ...  # static # readonly
        WithoutIonRTProfiles: BasicTypes.MfeCompound.InfoLevel = (
            ...
        )  # static # readonly
        WithoutIsotopeClusterInfo: BasicTypes.MfeCompound.InfoLevel = (
            ...
        )  # static # readonly

    class SortingType(
        System.IConvertible, System.IComparable, System.IFormattable
    ):  # Struct
        ByMass: BasicTypes.MfeCompound.SortingType = ...  # static # readonly
        ByRT: BasicTypes.MfeCompound.SortingType = ...  # static # readonly
        ByVolume: BasicTypes.MfeCompound.SortingType = ...  # static # readonly

class ModificationSpecies(IModificationSpecies):  # Class
    @overload
    def __init__(self) -> None: ...
    @overload
    def __init__(
        self, baseCount: int, modifications: BasicTypes.ModificationUnitCollection
    ) -> None: ...
    @overload
    def __init__(
        self, modificationUnit: str, zPerUnit: int, unitCount: int
    ) -> None: ...
    @overload
    def __init__(self, modificationUnit: IModificationUnit, unitCount: int) -> None: ...
    @overload
    def __init__(self, oldStyle: IIonSpeciesDetails) -> None: ...

    ElectronGain: IModificationSpecies  # static
    ElectronLoss: IModificationSpecies  # static
    ProtonGain: IModificationSpecies  # static
    ProtonLoss: IModificationSpecies  # static
    UnknownSpeciesMessage: str  # static # readonly

    BaseCount: int  # readonly
    IsInUndeterminedState: bool  # readonly
    Modifications: Dict[IModificationUnit, int]  # readonly

    @staticmethod
    def ParseCefFormat(
        CefFormat: str, z: int, isotopeIndex: int
    ) -> IModificationSpecies: ...
    def BinaryWrite(self, writer: System.IO.BinaryWriter) -> None: ...
    @overload
    @staticmethod
    def CreateDefaultIonSpecies(
        z: int, ionSource: SpectrumSetMetadata.MSSourceType
    ) -> IModificationSpecies: ...
    @overload
    @staticmethod
    def CreateDefaultIonSpecies(
        z: int, chargeType: BasicTypes.IonizationChargeType
    ) -> IModificationSpecies: ...
    def BinaryRead(self, reader: System.IO.BinaryReader) -> None: ...

class ModificationUnitCollection:  # Class
    def __init__(self) -> None: ...
    def Add(self, unit: IModificationUnit, count: int) -> None: ...

class OrderedXY:  # Class
    def __init__(self) -> None: ...

    Length: int  # readonly
    SummedIntensity: float  # readonly
    XRange: RangeDouble  # readonly

    def GetIndexLeftOf(self, x: float) -> int: ...
    def GetX(self, index: int) -> float: ...
    def GetY(self, index: int) -> float: ...
    def GetNearestIndex(self, x: float) -> int: ...
    def SumIntensity(self, range: RangeDouble, intialStartingIndex: int) -> float: ...

class ParentFragmentData:  # Class
    @overload
    def __init__(
        self,
        parentLowestMZ: float,
        parentZ: int,
        deisotopedFragmentPeaks: List[SpectralPeak],
        fragmentZs: List[int],
        parameter: BasicTypes.ParentFragmentData.Parameter,
    ) -> None: ...
    @overload
    def __init__(
        self,
        parentClusterPeaks: List[SpectralPeak],
        parentZ: int,
        fragmentSpectra: List[SpectralPeak],
        parameter: BasicTypes.ParentFragmentData.Parameter,
    ) -> None: ...
    @overload
    def __init__(
        self,
        parentMz: float,
        parentZ: int,
        fragmentSpectra: List[SpectralPeak],
        parameter: BasicTypes.ParentFragmentData.Parameter,
    ) -> None: ...
    @overload
    def __init__(
        self,
        parentZ: int,
        fragmentSpectra: List[SpectralPeak],
        parameter: BasicTypes.ParentFragmentData.Parameter,
        parentMz: float,
    ) -> None: ...

    FragmentIons: List[BasicTypes.FragmentIon]  # readonly

    @staticmethod
    def FilterByIntensity(
        ions: List[BasicTypes.FragmentIon], maxIonToKeep: int
    ) -> List[BasicTypes.FragmentIon]: ...
    @staticmethod
    def GetDeisotopedData(
        productPeaks: List[SpectralPeak],
        productMassAccuray: List[float],
        precursorMZ: float,
        precursorZ: int,
        parameter: BasicTypes.ParentFragmentData.Parameter,
    ) -> BasicTypes.ParentFragmentData: ...

    # Nested Types

    class Parameter:  # Class
        def __init__(self) -> None: ...

        FragmentInfo: BasicTypes.FragmentSpectraInfo
        ParentMassAccurancyCoefficients: List[float]

        def FromXml(self, siblings: System.Xml.XmlNodeList) -> None: ...
        def ToXml(self, doc: System.Xml.XmlDocument) -> System.Xml.XmlElement: ...
        def Clone(self) -> BasicTypes.ParentFragmentData.Parameter: ...

class Peak:  # Class
    @overload
    def __init__(self) -> None: ...
    @overload
    def __init__(
        self, loc: float, height: float, qualityCode: BasicTypes.Peak.Tag
    ) -> None: ...

    Height: float
    Intensity: float  # readonly
    Location: float  # readonly
    Overlapping: bool  # readonly
    QualityCode: BasicTypes.Peak.Tag  # readonly
    Saturated: bool  # readonly

    @staticmethod
    def CodeForSaturated(code: BasicTypes.Peak.Tag) -> bool: ...
    def CreatePeakAt(self, location: float) -> BasicTypes.Peak: ...
    @staticmethod
    def CodeForOverlapping(code: BasicTypes.Peak.Tag) -> bool: ...
    def PutTag(self, tag: BasicTypes.Peak.Tag) -> None: ...
    def RemoveTag(self, tag: BasicTypes.Peak.Tag) -> None: ...

    # Nested Types

    class Tag(System.IConvertible, System.IComparable, System.IFormattable):  # Struct
        Forbiden: BasicTypes.Peak.Tag = ...  # static # readonly
        MzOverlapping: BasicTypes.Peak.Tag = ...  # static # readonly
        Normal: BasicTypes.Peak.Tag = ...  # static # readonly
        PotentialDetectorRing: BasicTypes.Peak.Tag = ...  # static # readonly
        Saturated: BasicTypes.Peak.Tag = ...  # static # readonly
        SoftwareIntroduced: BasicTypes.Peak.Tag = ...  # static # readonly
        StatisticsOnConstituentNeeded: BasicTypes.Peak.Tag = ...  # static # readonly

class Peak2D(IIsotope, BasicTypes.Peak):  # Class
    @overload
    def __init__(self) -> None: ...
    @overload
    def __init__(
        self,
        loc: float,
        alternativeLocation: float,
        height: float,
        errorCode: BasicTypes.Peak.Tag,
    ) -> None: ...
    @overload
    def __init__(self, p: BasicTypes.Peak2D, abundanceScale: float) -> None: ...

    AlternativeLocation: float  # readonly
    Area: float

    def SetLocation(self, value_: float) -> None: ...
    def CreatePeakAt(self, location: float) -> BasicTypes.Peak: ...

class Peak3D(
    BasicTypes.Peak2D,
    BasicTypes.ILocation2D,
    BasicTypes.IIntensity2D,
    System.IComparable,
    IIsotope,
    BasicTypes.IChromatographyObject,
):  # Class
    @overload
    def __init__(self, scanWidths: List[float]) -> None: ...
    @overload
    def __init__(
        self, signals: List[BasicTypes.Signal3D], scanWidths: List[float]
    ) -> None: ...
    @overload
    def __init__(
        self,
        mz: float,
        rt: float,
        volume: float,
        rtWidth: float,
        scanWidths: List[float],
    ) -> None: ...
    @overload
    def __init__(self, mz: float, rt: float, volume: float, height: float) -> None: ...
    @overload
    def __init__(self, p: BasicTypes.Peak3D) -> None: ...
    @overload
    def __init__(self, p: BasicTypes.Peak3D, abundanceScale: float) -> None: ...

    Abundance: float  # readonly
    Centroid: Vector  # readonly
    Gaussianness: float  # readonly
    Height: float  # readonly
    Intensity: float  # readonly
    LocalEIC: List[float]  # readonly
    Location: float  # readonly
    MZ: float  # readonly
    MZBounds: RangeDouble  # readonly
    MaxHeight: float  # readonly
    NewRtWidth: float  # readonly
    PotentialOtherHalf: BasicTypes.Peak3D  # readonly
    QScore: float
    QualityCode: BasicTypes.Peak.Tag  # readonly
    QualityScore: float
    RT: float  # readonly
    RTBounds: RangeDouble  # readonly
    RTSolidness: float  # readonly
    RTWidth: float  # readonly
    RetentionTime: float  # readonly
    RetentionTimePeakWidth: float  # readonly
    SNRatio: float
    SaturateScanRange: RangeInt  # readonly
    ScanBounds: RangeInt  # readonly
    ScanIndexAtApex: int  # readonly
    ScanIndexAtCentroid: int  # readonly
    ScanWidths: List[float]  # readonly
    ShapeQuality: float  # readonly
    Signal3Ds: List[ISignal3D]  # readonly
    Signals: List[BasicTypes.Signal3D]
    SpanOfScan: int  # readonly
    Volume: float
    XLocation: float  # readonly
    YLocation: float  # readonly

    def CalculateRTTruncationBias(self) -> float: ...
    @staticmethod
    def AreaGap(pk1: BasicTypes.Peak3D, pk2: BasicTypes.Peak3D) -> float: ...
    @staticmethod
    def Split(
        localDividingIndex: int,
        pk: BasicTypes.Peak3D,
        questionableSplit: bool,
        peak1: BasicTypes.Peak3D,
        peak2: BasicTypes.Peak3D,
    ) -> None: ...
    def RefineMass(self) -> None: ...
    def RefineRT(self) -> None: ...
    def CalculateRTTruncationIndex(self) -> float: ...
    def GetMzAtScan(self, scanIndex: int) -> float: ...
    def Merge(self, pk: BasicTypes.Peak3D) -> None: ...
    def IsSpiky(self, snRatio: float) -> bool: ...
    @staticmethod
    def GetRtDifferenceOfApexes(
        p1: BasicTypes.Peak3D, p2: BasicTypes.Peak3D
    ) -> float: ...
    def GetHeightAtScan(self, scanIndex: int) -> float: ...
    @overload
    @staticmethod
    def CalculateRTSolidness(startPoint: int, endPoint: int) -> float: ...
    @overload
    @staticmethod
    def CalculateRTSolidness(
        eic: List[float], startPoint: int, endPoint: int
    ) -> float: ...
    def ResetSignals(self, signals: List[BasicTypes.Signal3D]) -> None: ...
    def GetRTTrimLines(self) -> List[int]: ...
    def ApplyTimeShiftCorrection(
        self, shiftModel: Regression, factor: float
    ) -> None: ...
    def SetSNForNoise(self, noise: float) -> None: ...
    def ResetData(
        self, signals: List[BasicTypes.Signal3D], scanWidths: List[float]
    ) -> None: ...
    def CalculateVolumeExcludingRange(self, excludedScanRange: RangeInt) -> float: ...
    @overload
    @staticmethod
    def CalculatedEicCorrelationCoefficient(
        p1: BasicTypes.Peak3D, p2: BasicTypes.Peak3D, overlappingPartCC: float
    ) -> float: ...
    @overload
    @staticmethod
    def CalculatedEicCorrelationCoefficient(
        r1: RangeInt,
        r2: RangeInt,
        eic1: List[float],
        eic2: List[float],
        overlappingPartCC: float,
    ) -> float: ...
    def TrimInRT(self, trimLines: List[int]) -> None: ...
    def DropRTProfile(self) -> None: ...
    def Clone(self) -> BasicTypes.Peak3D: ...
    def CompareTo(self, obj: Any) -> int: ...
    def GetAverageScanWidth(self) -> float: ...
    def SetQualityCode(self, qualityCode: BasicTypes.Peak.Tag) -> None: ...

    # Nested Types

    class Comparer(
        System.Collections.Generic.IComparer[BasicTypes.Peak3D],
        System.Collections.IComparer,
    ):  # Class
        @overload
        def __init__(self, type: BasicTypes.Peak3D.Comparer.Type) -> None: ...
        @overload
        def __init__(
            self, type: BasicTypes.Peak3D.Comparer.Type, asc: bool
        ) -> None: ...
        @overload
        def Compare(self, o1: Any, o2: Any) -> int: ...
        @overload
        def Compare(self, p1: BasicTypes.Peak3D, p2: BasicTypes.Peak3D) -> int: ...

        # Nested Types

        class Type(
            System.IConvertible, System.IComparable, System.IFormattable
        ):  # Struct
            ByHight: BasicTypes.Peak3D.Comparer.Type = ...  # static # readonly
            ByM_z: BasicTypes.Peak3D.Comparer.Type = ...  # static # readonly
            ByRT: BasicTypes.Peak3D.Comparer.Type = ...  # static # readonly
            ByVolume: BasicTypes.Peak3D.Comparer.Type = ...  # static # readonly

class PeakList(Iterable[Any], BasicTypes.OrderedXY, BasicTypes.ISpectrum):  # Class
    @overload
    def __init__(self) -> None: ...
    @overload
    def __init__(self, size: int, rt: float) -> None: ...
    @overload
    def __init__(self, peaks: List[SpectralPeak]) -> None: ...

    Calib: BasicTypes.TOFCalib
    def __getitem__(self, i: int) -> BasicTypes.Peak: ...
    Length: int  # readonly
    LocationSpan: float  # readonly
    MaxHeight: float  # readonly
    Peaks: System.Collections.ArrayList  # readonly
    RT: float
    SignalCount: int  # readonly
    SummedHeight: float  # readonly

    def GetEnumerator(self) -> Iterator[Any]: ...
    def Sort(self, type: BasicTypes.PeakList.SortingType) -> None: ...
    @overload
    def Add(self, p: BasicTypes.Peak) -> None: ...
    @overload
    def Add(self, peaks: BasicTypes.PeakList) -> None: ...
    def TrimToSize(self) -> None: ...
    def GetImage(self, binRange: RangeInt) -> List[float]: ...
    def SetBins(self, binStarts: List[float]) -> None: ...
    def GetX(self, index: int) -> float: ...
    def GetY(self, index: int) -> float: ...
    def AddRange(self, peaks: List[Any]) -> None: ...
    def KeepRange(self, startIndex: int, count: int) -> None: ...
    def Remove(self, pk: BasicTypes.Peak) -> None: ...
    def GetDifference(self, subList: BasicTypes.PeakList) -> BasicTypes.PeakList: ...
    def CalculateSummmedHeight(
        self, xRange: RangeDouble, searchStartSignalIndex: int
    ) -> float: ...

    # Nested Types

    class Comparer(
        System.Collections.Generic.IComparer[BasicTypes.Peak],
        System.Collections.IComparer,
    ):  # Class
        def __init__(self, type: BasicTypes.PeakList.SortingType) -> None: ...
        @overload
        def Compare(self, o1: Any, o2: Any) -> int: ...
        @overload
        def Compare(self, p1: BasicTypes.Peak, p2: BasicTypes.Peak) -> int: ...

    class SortingType(
        System.IConvertible, System.IComparable, System.IFormattable
    ):  # Struct
        ByIntensity: BasicTypes.PeakList.SortingType = ...  # static # readonly
        ByLocation: BasicTypes.PeakList.SortingType = ...  # static # readonly

class PrecursorDataSet(BasicTypes.IPrecursorDataSet):  # Class
    def __init__(
        self,
        mzOfInterest: float,
        precursorZ: int,
        zValueNeedsConfirm: bool,
        precursorIonSpecies: IModificationSpecies,
        isolationWindow: IsolationWindowType,
        EIData: bool,
    ) -> None: ...

    IonSpecies: IModificationSpecies  # readonly
    IsolationWindow: IsolationWindowType  # readonly
    PrecursorFullSaturation: float  # readonly
    PrecursorIonSpecies: IModificationSpecies  # readonly
    PrecursorIsotopePeaks: List[SpectralPeak]  # readonly
    PrecursorMz: float  # readonly

    def GetPrecursorZ(self, zValueNeedsConfirm: bool) -> int: ...
    def GetCollisionEnergies(self) -> Sequence[float]: ...
    def GetProductSpectraFullSaturation(self, collisionEnergy: float) -> float: ...
    def AddProductLineSpectrum(
        self,
        collisionEnergy: float,
        spectrum: List[SpectralPeak],
        timeRange: RangeDouble,
        fullSaturation: float,
        addDuplication: bool,
    ) -> None: ...
    def GetProductSpectrum(self, collisionEnergy: float) -> List[SpectralPeak]: ...
    def ConfirmZValue(self) -> None: ...
    def GetProductCompositeSpectrum(
        self,
        scheme: CeCompositeSpectraMakingScheme,
        productMassErrorCoefficients: List[float],
    ) -> List[SpectralPeak]: ...

class PrecursorIsotopeClusterExtractor:  # Class
    def __init__(
        self, precursorMassCoefficients: List[float], character: IsotopeCharacter
    ) -> None: ...
    def GetIsotopePeaks(
        self, precursorMZ: float, precursorZ: int, rawPeaks: List[SpectralPeak]
    ) -> List[SpectralPeak]: ...

class Protein(
    IMass_TimeSeparatedObject,
    Iterable[Any],
    IProtein,
    BasicTypes.ILadder,
    BasicTypes.IChromatographyObject,
    IFilterable,
):  # Class
    @overload
    def __init__(self, polarity: SpectrumSetMetadata.MSPolarity) -> None: ...
    @overload
    def __init__(
        self,
        polarity: SpectrumSetMetadata.MSPolarity,
        peak1: BasicTypes.Peak,
        peak2: BasicTypes.Peak,
        chargeCount: int,
    ) -> None: ...
    @overload
    def __init__(self, undertone: BasicTypes.Protein, undertoneFactor: int) -> None: ...
    @overload
    def __init__(
        self, polarity: SpectrumSetMetadata.MSPolarity, peak: BasicTypes.Peak
    ) -> None: ...
    @overload
    def __init__(
        self,
        rt: float,
        mass: float,
        volume: float,
        chargeStates: List[BasicTypes.Protein.ChargeState],
    ) -> None: ...

    Abundance: float
    ChargeStateSpan: int  # readonly
    ChargeStates: List[BasicTypes.Protein.ChargeState]
    EmptyStateCountAtBeginning: int  # readonly
    EmptyStateCountAtEnd: int  # readonly
    GapCount: int  # readonly
    Height: float  # readonly
    IonSet: List[IChargeState]  # readonly
    Mass: float
    MassStandardDeviation: float  # readonly
    MaxChargeCount: int  # readonly
    MaxHeight: float  # readonly
    MinChargeCount: int  # readonly
    NetProtonMass: float  # readonly
    NextPeakChargeCount: int  # readonly
    OverlappingPeakCount: int  # readonly
    PeakCount: int  # readonly
    PeakList: BasicTypes.PeakList  # readonly
    QScore: float
    QualityScore: float
    RT: float
    RTDeviation: float  # readonly
    RetentionTime: float  # readonly
    RetentionTimePeakWidth: float  # readonly
    Saturated: bool  # readonly

    def GetEnumerator(self) -> Iterator[Any]: ...
    def Add(self, p: BasicTypes.Peak) -> None: ...
    def Merge(self, another: BasicTypes.Protein) -> None: ...
    @staticmethod
    def LocationError(
        mzError: BasicTypes.ChromMetadata.MZError, mz: float
    ) -> float: ...
    def AddLast(self, p: BasicTypes.Peak) -> None: ...
    def TrimEmptyStatesAtEnds(self) -> None: ...
    @staticmethod
    def PredictIntensityRangeByExpolating(
        intensity1: float,
        intensity2: float,
        chargeCount1: int,
        chargeCount2: int,
        chargeCount: int,
    ) -> RangeDouble: ...
    @staticmethod
    def DeterminNetProtonMass(polarity: SpectrumSetMetadata.MSPolarity) -> float: ...
    @staticmethod
    def SortListBy(
        proteins: List[BasicTypes.Protein],
        sortingType: BasicTypes.MfeCompound.SortingType,
        ascending: bool,
    ) -> None: ...
    def AddFirst(self, p: BasicTypes.Peak) -> None: ...
    @staticmethod
    def PredictIntensityRangeByIntrapolating(
        intensity1: float,
        intensity2: float,
        chargeCount1: int,
        chargeCount2: int,
        chargeCount: int,
    ) -> RangeDouble: ...

    # Nested Types

    class ChargeState(IChargeState):  # Class
        def __init__(self, z: int, peak: BasicTypes.Peak) -> None: ...

        Abundance: float  # readonly
        ChargeCount: int  # readonly
        HasPeak: bool  # readonly
        MZ: float  # readonly
        MaxHeight: float  # readonly
        Peak: BasicTypes.Peak  # readonly
        PredictedMass: float  # readonly
        RetentionTime: float  # readonly
        Saturated: bool  # readonly
        Signal3Ds: List[ISignal3D]  # readonly
        Z: int  # readonly

    class Comparer(
        System.Collections.Generic.IComparer[BasicTypes.Protein],
        System.Collections.IComparer,
    ):  # Class
        def __init__(
            self, type: BasicTypes.MfeCompound.SortingType, ascending: bool
        ) -> None: ...
        @overload
        def Compare(self, o1: Any, o2: Any) -> int: ...
        @overload
        def Compare(self, c1: BasicTypes.Protein, c2: BasicTypes.Protein) -> int: ...

class QualSpeciesExpressionUtilities:  # Class
    def __init__(self) -> None: ...
    @staticmethod
    def ParseMultimerNumberAndAdducts(
        s: str, multimerNumber: int, adducts: str
    ) -> None: ...
    @overload
    @staticmethod
    def Parse(
        expression: str, multimerNumber: int, adducts: str, neutralModification: str
    ) -> None: ...
    @overload
    @staticmethod
    def Parse(expression: str) -> IModificationSpecies: ...
    @overload
    @staticmethod
    def Parse(complexFormula: str, z: int, unitCount: int) -> IModificationUnit: ...
    @staticmethod
    def FormatQualSpecies(species: IModificationSpecies) -> str: ...

class SampleChemistryInfo:  # Class
    def __init__(self) -> None: ...

    Averagine: bool
    MaxChargeCount: int

    CombinedAdducts: System.Collections.Generic.List[IModificationUnit]  # readonly
    NegativeAdducts: System.Collections.Generic.List[IModificationUnit]  # readonly
    PositiveAdducts: System.Collections.Generic.List[IModificationUnit]  # readonly
    SaltDominated: bool  # readonly

    @overload
    def Equals(self, p: BasicTypes.SampleChemistryInfo) -> bool: ...
    @overload
    def Equals(self, obj: Any) -> bool: ...
    def SetPositiveAdducts(self, adducts: List[str], saltDominated: bool) -> None: ...
    def GetAdductsForPolarity(
        self, polarity: SpectrumSetMetadata.MSPolarity
    ) -> System.Collections.Generic.List[IModificationUnit]: ...
    def SetNegativeAdducts(self, adducts: List[str]) -> None: ...
    @staticmethod
    def Validate(adducts: List[IModificationUnit]) -> str: ...
    def GetHashCode(self) -> int: ...
    @overload
    def SetAdducts(
        self,
        saltDominated: bool,
        positiveAdducts: System.Collections.Generic.List[IModificationUnit],
        negativeAdducts: System.Collections.Generic.List[IModificationUnit],
    ) -> None: ...
    @overload
    def SetAdducts(
        self, saltDominated: bool, combinedAdducts: List[IModificationUnit]
    ) -> None: ...
    def Clone(self) -> BasicTypes.SampleChemistryInfo: ...

class Signal3D(ISignal3D, ClusteringSequential.IElement):  # Class
    @overload
    def __init__(self, sig: BasicTypes.Signal3D) -> None: ...
    @overload
    def __init__(self, sig: BasicTypes.Signal3D, abundanceScale: float) -> None: ...
    @overload
    def __init__(
        self,
        hight: float,
        mz: float,
        time: float,
        rtIndex: int,
        qualityCode: BasicTypes.Peak.Tag,
    ) -> None: ...

    m_mz: float

    Height: float
    Intensity: float  # readonly
    MZ: float  # readonly
    Noise: float
    Overlapping: bool  # readonly
    QualityCode: BasicTypes.Peak.Tag  # readonly
    RT: float
    RetentionTime: float  # readonly
    Saturated: bool  # readonly
    ScanIndex: int  # readonly

    # Nested Types

    class Comparer(
        System.Collections.IComparer,
        System.Collections.Generic.IComparer[BasicTypes.Signal3D],
    ):  # Class
        def __init__(
            self, type: BasicTypes.Signal3D.Comparer.Type, ascent: bool
        ) -> None: ...
        @overload
        def Compare(self, o1: Any, o2: Any) -> int: ...
        @overload
        def Compare(self, s1: BasicTypes.Signal3D, s2: BasicTypes.Signal3D) -> int: ...

        # Nested Types

        class Type(
            System.IConvertible, System.IComparable, System.IFormattable
        ):  # Struct
            ByHight: BasicTypes.Signal3D.Comparer.Type = ...  # static # readonly
            ByM_z: BasicTypes.Signal3D.Comparer.Type = ...  # static # readonly
            ByRT: BasicTypes.Signal3D.Comparer.Type = ...  # static # readonly

class TOFCalib:  # Class
    def __init__(
        self,
        polynomialCoeff: List[float],
        minCalibratedTime: float,
        maxCalibratedTime: float,
    ) -> None: ...

    Coeffiecients: List[float]  # readonly

    def GetTime(self, mz: float) -> float: ...
    def GetMZ(self, t: float) -> float: ...
    @staticmethod
    def GetCalibreatedRange(metadata: SpectrumSetMetadata) -> RangeDouble: ...
    def Clone(self) -> BasicTypes.TOFCalib: ...
    def Derivative(self, t: float) -> float: ...

class XYCollection:  # Class
    def __init__(self, metaData: BasicTypes.ChromMetadata) -> None: ...

    AverageSpectrum: BasicTypes.ISpectrum  # readonly
    BinCount: int  # readonly
    Metadata: BasicTypes.ChromMetadata  # readonly
    RTRange: RangeDouble  # readonly
    ScanCount: int  # readonly
    ScanTimes: List[float]  # readonly
    SpectralXRange: RangeDouble  # readonly
    TIC: BasicTypes.Chromatogram  # readonly
    TOFCalibrations: List[BasicTypes.TOFCalib]  # readonly

    @overload
    def GetEIC(
        self, mzRange: RangeDouble, binChoice: BasicTypes.XYCollection.EICBinChoice
    ) -> BasicTypes.Chromatogram: ...
    @overload
    def GetEIC(self, mz: float) -> BasicTypes.Chromatogram: ...
    def ApplyTimeShiftCorrection(
        self, shiftModel: Regression, factor: float
    ) -> None: ...
    @overload
    def GetSpectrum(
        self, scanRange: RangeInt, type: BasicTypes.XYCollection.SpectrumType
    ) -> BasicTypes.ISpectrum: ...
    @overload
    def GetSpectrum(self, scan: int) -> BasicTypes.ISpectrum: ...
    @overload
    def GetSpectrum(self, rt: float) -> BasicTypes.ISpectrum: ...
    def GetAverageSpectrum(self, rtRange: RangeDouble) -> BasicTypes.ISpectrum: ...
    def GetNearestScan(self, rt: float) -> int: ...
    def GetTOFCalib(self, rt: float) -> BasicTypes.TOFCalib: ...
    def GetSpectralLoc(self, bin: int) -> float: ...
    @overload
    def GetSummedSpectrum(
        self, rt: float, rtPeakWidth: float
    ) -> BasicTypes.ISpectrum: ...
    @overload
    def GetSummedSpectrum(self, rtRange: RangeDouble) -> BasicTypes.ISpectrum: ...

    # Nested Types

    class EICBinChoice(
        System.IConvertible, System.IComparable, System.IFormattable
    ):  # Struct
        FillBins: BasicTypes.XYCollection.EICBinChoice = ...  # static # readonly
        FillWhenCentroidCovered: BasicTypes.XYCollection.EICBinChoice = (
            ...
        )  # static # readonly
        NoBinning: BasicTypes.XYCollection.EICBinChoice = ...  # static # readonly

    class SpectrumType(
        System.IConvertible, System.IComparable, System.IFormattable
    ):  # Struct
        Averaged: BasicTypes.XYCollection.SpectrumType = ...  # static # readonly
        Summed: BasicTypes.XYCollection.SpectrumType = ...  # static # readonly

class XYCollectionLine(BasicTypes.XYCollection):  # Class
    @overload
    def __init__(self, metaData: BasicTypes.ChromMetadata) -> None: ...
    @overload
    def __init__(
        self,
        metaData: BasicTypes.ChromMetadata,
        peaks: List[BasicTypes.Peak3D],
        scanTimes: List[float],
        shiftModel: Regression,
        factor: float,
    ) -> None: ...
    @overload
    def __init__(
        self, metaData: BasicTypes.ChromMetadata, compounds: List[ICompound]
    ) -> None: ...
    @overload
    def __init__(
        self, template: BasicTypes.XYCollectionLine, peaks: List[BasicTypes.Peak3D]
    ) -> None: ...
    @overload
    def __init__(
        self,
        template: BasicTypes.XYCollectionLine,
        peakLists: System.Collections.Generic.List[
            System.Collections.Generic.List[BasicTypes.Peak]
        ],
    ) -> None: ...
    @overload
    def __init__(
        self,
        template: BasicTypes.XYCollectionLine,
        groups: List[BasicTypes.CoelutionGroup],
        shiftModel: Regression,
        factor: float,
    ) -> None: ...

    BinCount: int  # readonly
    IsBinned: bool  # readonly
    PeakLists: List[BasicTypes.PeakList]  # readonly
    TheEICCollection: BasicTypes.EICCollection

    @overload
    def GetEIC(
        self, mzRange: RangeDouble, binChoice: BasicTypes.XYCollection.EICBinChoice
    ) -> BasicTypes.Chromatogram: ...
    @overload
    def GetEIC(self, mz: float) -> BasicTypes.Chromatogram: ...
    def FindResolutionBinRange(self, spectralXRange: RangeDouble) -> RangeInt: ...
    @overload
    def Add(self, peaks: BasicTypes.PeakList, massRange: RangeDouble) -> None: ...
    @overload
    def Add(self, peaks: BasicTypes.PeakList) -> None: ...
    def GetSpectrum(
        self, scanRange: RangeInt, type: BasicTypes.XYCollection.SpectrumType
    ) -> BasicTypes.ISpectrum: ...
    def TrimToSize(self) -> None: ...
    def GetAllSignal3D(
        self,
    ) -> System.Collections.Generic.List[BasicTypes.Signal3D]: ...
    def FindResolutionBin(self, mz: float) -> int: ...
    def GetSpectralLoc(self, bin: int) -> float: ...
    def GetScanImages(
        self, scanRange: RangeInt, binRange: RangeInt
    ) -> List[List[float]]: ...

class XYCollectionProfile(BasicTypes.XYCollection):  # Class
    @overload
    def __init__(
        self, metadata: BasicTypes.ChromMetadata, scanCount: int, mzBinWidth: float
    ) -> None: ...
    @overload
    def __init__(
        self,
        metadata: BasicTypes.ChromMetadata,
        scanCount: int,
        binCount: int,
        mzBinWidth: float,
    ) -> None: ...
    @overload
    def __init__(self, metaData: BasicTypes.ChromMetadata) -> None: ...
    @overload
    def __init__(
        self, metadata: BasicTypes.ChromMetadata, mzBinWidth: float
    ) -> None: ...

    BinCount: int  # readonly
    MZBinWidth: float

    @overload
    def GetEIC(
        self, mzRange: RangeDouble, binChoice: BasicTypes.XYCollection.EICBinChoice
    ) -> BasicTypes.Chromatogram: ...
    @overload
    def GetEIC(self, mz: float) -> BasicTypes.Chromatogram: ...
    def GetSpectrum(
        self, scanRange: RangeInt, type: BasicTypes.XYCollection.SpectrumType
    ) -> BasicTypes.ISpectrum: ...
    @overload
    def SetData(
        self,
        scan: int,
        rt: float,
        rawData: List[int],
        x0: float,
        calib: BasicTypes.TOFCalib,
    ) -> int: ...
    @overload
    def SetData(
        self,
        scan: int,
        rt: float,
        calib: BasicTypes.TOFCalib,
        x0: float,
        lowSignals: List[int],
        mediumSignals: List[int],
        highSignals: List[int],
    ) -> None: ...
    def GetScans(self, scanRange: RangeInt) -> List[List[int]]: ...
    def GetSpectralLoc(self, bin: int) -> float: ...



# -*- coding: utf-8 -*-
import typing

# Import specific members from typing used in hints
from typing import (
    Any,
    Callable,
    Dict,
    FrozenSet,
    Generic,
    Iterable,
    Iterator,
    List,
    Mapping,
    Optional,
    Sequence,
    Set,
    Tuple,
    TypeVar,
    Union,
    overload,
)

import datetime
from enum import Enum

from mh_operator.SDK import Agilent, System

from .AppCommand import ApplicationType
from .UI import UIContext, ViewMode
from .Utils import IImportCompounds

# Stubs for namespace: Agilent.MassSpectrometry.DataAnalysis.Quantitative.ReportMethodEdit.ScriptIF

class IEditView(object):  # Interface
    CanCopy: bool  # readonly
    CanCut: bool  # readonly
    CanDelete: bool  # readonly
    CanPaste: bool  # readonly
    CanRedo: Optional[bool]  # readonly
    CanUndo: Optional[bool]  # readonly

    def Redo(self) -> None: ...
    def Undo(self) -> None: ...
    def Copy(self) -> None: ...
    def Cut(self) -> None: ...
    def Paste(self) -> None: ...
    def Delete(self) -> None: ...

class IMainForm(System.Windows.Forms.IWin32Window):  # Interface
    CurrentView: (
        Agilent.MassSpectrometry.DataAnalysis.Quantitative.ReportMethodEdit.ScriptIF.IEditView
    )  # readonly
    IsInEditMode: bool  # readonly
    TemplatesView: (
        Agilent.MassSpectrometry.DataAnalysis.Quantitative.ReportMethodEdit.ScriptIF.ITemplatesView
    )  # readonly
    ViewMode: ViewMode

class ITemplatesView(
    Agilent.MassSpectrometry.DataAnalysis.Quantitative.ReportMethodEdit.ScriptIF.IEditView
):  # Interface
    def CanRemoveTemplates(self) -> bool: ...
    def RemoveTemplates(self) -> None: ...

class IUIState(object):  # Interface
    ApplicationType: ApplicationType  # readonly
    CanRedo: bool  # readonly
    CanUndo: bool  # readonly
    ImportCompounds: IImportCompounds  # readonly
    IsDirty: bool  # readonly
    MainForm: (
        Agilent.MassSpectrometry.DataAnalysis.Quantitative.ReportMethodEdit.ScriptIF.IMainForm
    )  # readonly
    PathName: str  # readonly
    _UIContext: UIContext  # readonly

    def ExecuteCommand(self, command: str, parameters: List[Any]) -> Any: ...
    def Redo(self) -> None: ...
    def Undo(self) -> None: ...
    def ExitApplication(self, forceExit: bool) -> None: ...



# -*- coding: utf-8 -*-
import typing

# Import specific members from typing used in hints
from typing import (
    Any,
    Callable,
    Dict,
    FrozenSet,
    Generic,
    Iterable,
    Iterator,
    List,
    Mapping,
    Optional,
    Sequence,
    Set,
    Tuple,
    TypeVar,
    Union,
    overload,
)

import datetime
from enum import Enum

from mh_operator.SDK import Agilent, System

from . import (
    AppCommandContext,
    ChromatographyType,
    CurveFit,
    DoubleRange,
    IntRange,
    QuantitationDataSet,
    SampleType,
)

# Stubs for namespace: Agilent.MassSpectrometry.DataAnalysis.Quantitative.DynamicQuant

class BatchFileInfo:  # Class
    BatchDataPath: str  # readonly
    BatchFileName: str  # readonly
    BatchID: int  # readonly
    BatchSamples: List[
        Agilent.MassSpectrometry.DataAnalysis.Quantitative.DynamicQuant.BatchSampleInfo
    ]  # readonly
    CalibrationLevelInfo: (
        Agilent.MassSpectrometry.DataAnalysis.Quantitative.DynamicQuant.CalibrationLevelInfo
    )  # readonly
    CorrelationWindow: float  # readonly
    HighestLevelSample: (
        Agilent.MassSpectrometry.DataAnalysis.Quantitative.DynamicQuant.BatchSampleInfo
    )  # readonly
    NSamples: int  # readonly
    NTargetCompounds: int  # readonly
    TargetCompounds: List[
        Agilent.MassSpectrometry.DataAnalysis.Quantitative.DynamicQuant.TargetCompoundInfo
    ]  # readonly

    def GetCalibrationSamplesByLevel(
        self,
    ) -> System.Collections.Generic.SortedList[
        float,
        System.Collections.Generic.List[
            Agilent.MassSpectrometry.DataAnalysis.Quantitative.DynamicQuant.BatchSampleInfo
        ],
    ]: ...
    def GetSampleByFileName(
        self, sampleFileName: str
    ) -> (
        Agilent.MassSpectrometry.DataAnalysis.Quantitative.DynamicQuant.BatchSampleInfo
    ): ...

class BatchGlobalFeatureAnalysis(
    Agilent.MassSpectrometry.DataAnalysis.Quantitative.DynamicQuant.IDynamicQuant
):  # Class
    @overload
    def __init__(self, batchDataPath: str, batchFileName: str) -> None: ...
    @overload
    def __init__(
        self,
        batchDataPath: str,
        batchFileName: str,
        dqParams: Agilent.MassSpectrometry.DataAnalysis.Quantitative.DynamicQuant.DynamicQuantParams,
    ) -> None: ...
    @overload
    def __init__(self, context: AppCommandContext) -> None: ...
    @overload
    def __init__(
        self,
        context: AppCommandContext,
        dqParams: Agilent.MassSpectrometry.DataAnalysis.Quantitative.DynamicQuant.DynamicQuantParams,
    ) -> None: ...

    BatchFileInfo: (
        Agilent.MassSpectrometry.DataAnalysis.Quantitative.DynamicQuant.BatchFileInfo
    )  # readonly
    DynamicQuantParameters: (
        Agilent.MassSpectrometry.DataAnalysis.Quantitative.DynamicQuant.DynamicQuantParams
    )  # readonly
    TargetCompoundIDs: List[int]  # readonly

    def GetDynamicTargetsInRange(
        self, compoundID: int, lowLevelConc: float, highLevelConc: float
    ) -> List[
        Agilent.MassSpectrometry.DataAnalysis.Quantitative.DynamicQuant.IDynamicQuantTarget
    ]: ...
    def IsTargetCompoundSaturated(self, compoundID: int) -> bool: ...
    def GetDynamicTargetIonGroups(
        self, compoundID: int
    ) -> List[
        Agilent.MassSpectrometry.DataAnalysis.Quantitative.DynamicQuant.ICrossSampleIonGroup
    ]: ...
    def FindCrossSampleIonGroups(self) -> None: ...
    def GetDynamicTargets(
        self, compoundID: int
    ) -> List[
        Agilent.MassSpectrometry.DataAnalysis.Quantitative.DynamicQuant.IDynamicQuantTarget
    ]: ...

    CrossSampleAnalysisDone: (
        Agilent.MassSpectrometry.DataAnalysis.Quantitative.DynamicQuant.BatchGlobalFeatureAnalysis.CrossSampleAnalysisFinishedEventHandler
    )  # Event
    CrossSampleAnalysisStarting: (
        Agilent.MassSpectrometry.DataAnalysis.Quantitative.DynamicQuant.BatchGlobalFeatureAnalysis.CrossSampleAnalysisStartingEventHandler
    )  # Event
    SampleProcessingDone: (
        Agilent.MassSpectrometry.DataAnalysis.Quantitative.DynamicQuant.BatchGlobalFeatureAnalysis.SampleProcessingDoneEventHandler
    )  # Event

    # Nested Types

    class CrossSampleAnalysisFinishedEventHandler(
        System.MulticastDelegate,
        System.ICloneable,
        System.Runtime.Serialization.ISerializable,
    ):  # Class
        def __init__(self, object: Any, method: System.IntPtr) -> None: ...
        def EndInvoke(self, result: System.IAsyncResult) -> None: ...
        def BeginInvoke(
            self,
            sender: Any,
            args: System.EventArgs,
            callback: System.AsyncCallback,
            object: Any,
        ) -> System.IAsyncResult: ...
        def Invoke(self, sender: Any, args: System.EventArgs) -> None: ...

    class CrossSampleAnalysisStartingEventArgs(System.EventArgs):  # Class
        NSamples: int  # readonly

    class CrossSampleAnalysisStartingEventHandler(
        System.MulticastDelegate,
        System.ICloneable,
        System.Runtime.Serialization.ISerializable,
    ):  # Class
        def __init__(self, object: Any, method: System.IntPtr) -> None: ...
        def EndInvoke(self, result: System.IAsyncResult) -> None: ...
        def BeginInvoke(
            self,
            sender: Any,
            args: Agilent.MassSpectrometry.DataAnalysis.Quantitative.DynamicQuant.BatchGlobalFeatureAnalysis.CrossSampleAnalysisStartingEventArgs,
            callback: System.AsyncCallback,
            object: Any,
        ) -> System.IAsyncResult: ...
        def Invoke(
            self,
            sender: Any,
            args: Agilent.MassSpectrometry.DataAnalysis.Quantitative.DynamicQuant.BatchGlobalFeatureAnalysis.CrossSampleAnalysisStartingEventArgs,
        ) -> None: ...

    class SampleProcessingDoneEventArgs(System.EventArgs):  # Class
        SampleFileName: str  # readonly
        SampleIndex: int  # readonly

    class SampleProcessingDoneEventHandler(
        System.MulticastDelegate,
        System.ICloneable,
        System.Runtime.Serialization.ISerializable,
    ):  # Class
        def __init__(self, object: Any, method: System.IntPtr) -> None: ...
        def EndInvoke(self, result: System.IAsyncResult) -> None: ...
        def BeginInvoke(
            self,
            sender: Any,
            args: Agilent.MassSpectrometry.DataAnalysis.Quantitative.DynamicQuant.BatchGlobalFeatureAnalysis.SampleProcessingDoneEventArgs,
            callback: System.AsyncCallback,
            object: Any,
        ) -> System.IAsyncResult: ...
        def Invoke(
            self,
            sender: Any,
            args: Agilent.MassSpectrometry.DataAnalysis.Quantitative.DynamicQuant.BatchGlobalFeatureAnalysis.SampleProcessingDoneEventArgs,
        ) -> None: ...

class BatchSampleInfo:  # Class
    BatchInfo: (
        Agilent.MassSpectrometry.DataAnalysis.Quantitative.DynamicQuant.BatchFileInfo
    )  # readonly
    CalLevelInfo: (
        Agilent.MassSpectrometry.DataAnalysis.Quantitative.DynamicQuant.CalibrationLevelInfo
    )  # readonly
    ChromatographyType: ChromatographyType  # readonly
    Index: int  # readonly
    LevelConcentration: float  # readonly
    LevelIndex: int  # readonly
    LevelName: str  # readonly
    SampleDataPath: str  # readonly
    SampleDataSet: QuantitationDataSet  # readonly
    SampleFileName: str  # readonly
    SampleID: int  # readonly
    SampleType: SampleType  # readonly

    def GetTargetCompoundRow(
        self, compoundID: int
    ) -> QuantitationDataSet.TargetCompoundRow: ...

class CalibrationLevelInfo:  # Class
    LevelCount: int  # readonly

    def GetLevelIndexByConcentration(self, levelConcentration: float) -> int: ...
    def GetConcentrationForSample(self, sampleFileName: str) -> float: ...
    def GetLevelIndexForSample(self, sampleFileName: str) -> int: ...

class ComponentIonFeature(
    Agilent.MassSpectrometry.DataAnalysis.Quantitative.DynamicQuant.IIonGroupFeature
):  # Class
    FeatureInfo: (
        Agilent.MassSpectrometry.DataAnalysis.Quantitative.DynamicQuant.FeatureInfo
    )  # readonly
    IsDerivedFromComponent: bool  # readonly
    IsUserDefinedTargetIon: bool  # readonly
    Sample: (
        Agilent.MassSpectrometry.DataAnalysis.Quantitative.DynamicQuant.BatchSampleInfo
    )  # readonly
    TargetIon: (
        Agilent.MassSpectrometry.DataAnalysis.Quantitative.DynamicQuant.ITargetIon
    )  # readonly

    # Nested Types

    class ReverseHeightComparer(
        System.Collections.Generic.IComparer[
            Agilent.MassSpectrometry.DataAnalysis.Quantitative.DynamicQuant.ComponentIonFeature
        ]
    ):  # Class
        def __init__(self) -> None: ...
        def Compare(
            self,
            a: Agilent.MassSpectrometry.DataAnalysis.Quantitative.DynamicQuant.ComponentIonFeature,
            b: Agilent.MassSpectrometry.DataAnalysis.Quantitative.DynamicQuant.ComponentIonFeature,
        ) -> int: ...

class CrossSampleIonGroup(
    Agilent.MassSpectrometry.DataAnalysis.Quantitative.DynamicQuant.ICrossSampleIonGroup
):  # Class
    AverageMz: float  # readonly
    CompoundName: str  # readonly
    CompoundRT: float  # readonly
    ConcentrationRange: Agilent.MassSpectrometry.DataAnalysis.DoubleRangeD  # readonly
    IsUserDefinedTargetIonGroup: bool  # readonly
    LevelCount: int  # readonly
    LinearFit: CurveFit  # readonly
    MaxDeviationFromScalingRatio: float  # readonly
    SampleAreaResponses: List[float]  # readonly
    SampleConcentrations: List[float]  # readonly
    SampleHeightResponses: List[float]  # readonly
    SampleIonMZs: List[float]  # readonly
    SampleIonRTs: List[float]  # readonly
    TargetCompoundID: int  # readonly
    TotalIonCount: int  # readonly
    TotalLevelCount: int  # readonly
    UnsaturatedIonCount: int  # readonly
    UnsaturatedLevelCount: int  # readonly
    UnsaturatedRange: Agilent.MassSpectrometry.DataAnalysis.DoubleRangeD  # readonly
    UserDefinedTargetIon: (
        Agilent.MassSpectrometry.DataAnalysis.Quantitative.DynamicQuant.ITargetIon
    )  # readonly

    def ContainsUnsaturatedIonInSample(self, sampleIndex: int) -> bool: ...
    def GetAverageAreaResponseAtLevel(self, levelIndex: int) -> float: ...
    def GetLevelIndexByLevelName(self, levelName: str) -> int: ...
    def GetSampleIndexListAtLevel(
        self, levelIndex: int
    ) -> System.Collections.Generic.List[int]: ...
    def ContainsUnsaturatedIonAtLevel(self, levelName: str) -> bool: ...
    def HasIonInSample(self, sampleIndex: int) -> bool: ...

class DynamicQuantParams:  # Class
    @overload
    def __init__(self, separationType: ChromatographyType) -> None: ...
    @overload
    def __init__(self, paramFile: str, separationType: ChromatographyType) -> None: ...

    CONFINE_TO_LINEAR_RANGE: bool = ...  # static # readonly
    CONTIGUOUS_LEVELS: bool = ...  # static # readonly
    DIMER_REJECTION_THRESHOLD: float = ...  # static # readonly
    FEATURE_GROUP_ALIGNMENT: (
        Agilent.MassSpectrometry.DataAnalysis.Quantitative.DynamicQuant.FeatureGroupAlignment
    ) = ...  # static # readonly
    MAX_SCALING_RATIO_DEVIATION: float = ...  # static # readonly
    MAX_SCALING_RATIO_DEVIATION2: float = ...  # static # readonly
    MIN_FIT_R2: float = ...  # static # readonly
    MIN_LEVELS_PER_ION_GROUP: int = ...  # static # readonly
    MIN_TARGET_IONS_PER_COMPOUND: int = ...  # static # readonly
    MZ_TOLERANCE_PPM: float = ...  # static # readonly
    QUANTIFIER_METRIC: (
        Agilent.MassSpectrometry.DataAnalysis.Quantitative.DynamicQuant.QuantifierMetricType
    ) = ...  # static # readonly

    ChromType: ChromatographyType  # readonly
    ConfineToLinearRange: bool
    ContiguousLevels: bool
    DimerRejectionThreshold: float
    FeatureGroupAlignment: (
        Agilent.MassSpectrometry.DataAnalysis.Quantitative.DynamicQuant.FeatureGroupAlignment
    )
    MaxScalingRatioDeviation: float
    MaxScalingRatioDeviation2: float
    MinFitR2: float
    MinLevelsPerIonGroup: int
    MinTargetIonsPerCompound: int
    MzTolerancePpm: float
    NumThreads: int
    QuantifierMetric: (
        Agilent.MassSpectrometry.DataAnalysis.Quantitative.DynamicQuant.QuantifierMetricType
    )
    RefLibraryPath: str
    UseDeconvolution: bool  # readonly
    UseReferenceSpectrum: bool  # readonly

class FeatureGroupAlignment(
    System.IConvertible, System.IComparable, System.IFormattable
):  # Struct
    Deconvolution: (
        Agilent.MassSpectrometry.DataAnalysis.Quantitative.DynamicQuant.FeatureGroupAlignment
    ) = ...  # static # readonly
    DeconvolutionAndReferenceSpectrum: (
        Agilent.MassSpectrometry.DataAnalysis.Quantitative.DynamicQuant.FeatureGroupAlignment
    ) = ...  # static # readonly
    ReferenceSpectrum: (
        Agilent.MassSpectrometry.DataAnalysis.Quantitative.DynamicQuant.FeatureGroupAlignment
    ) = ...  # static # readonly

class FeatureInfo:  # Class
    ...

class ICrossSampleIonGroup(object):  # Interface
    AverageMz: float  # readonly
    CompoundName: str  # readonly
    CompoundRT: float  # readonly
    ConcentrationRange: Agilent.MassSpectrometry.DataAnalysis.DoubleRangeD  # readonly
    IsUserDefinedTargetIonGroup: bool  # readonly
    LevelCount: int  # readonly
    LinearFit: CurveFit  # readonly
    MaxDeviationFromScalingRatio: float  # readonly
    SampleAreaResponses: List[float]  # readonly
    SampleConcentrations: List[float]  # readonly
    SampleHeightResponses: List[float]  # readonly
    SampleIonMZs: List[float]  # readonly
    SampleIonRTs: List[float]  # readonly
    TargetCompoundID: int  # readonly
    TotalIonCount: int  # readonly
    TotalLevelCount: int  # readonly
    UnsaturatedIonCount: int  # readonly
    UnsaturatedLevelCount: int  # readonly
    UnsaturatedRange: Agilent.MassSpectrometry.DataAnalysis.DoubleRangeD  # readonly
    UserDefinedTargetIon: (
        Agilent.MassSpectrometry.DataAnalysis.Quantitative.DynamicQuant.ITargetIon
    )  # readonly

    def ContainsUnsaturatedIonInSample(self, sampleIndex: int) -> bool: ...
    def GetAverageAreaResponseAtLevel(self, levelIndex: int) -> float: ...
    def GetLevelIndexByLevelName(self, levelName: str) -> int: ...
    def GetSampleIndexListAtLevel(
        self, levelIndex: int
    ) -> System.Collections.Generic.List[int]: ...
    def ContainsUnsaturatedIonAtLevel(self, levelName: str) -> bool: ...
    def HasIonInSample(self, sampleIndex: int) -> bool: ...

class IDynamicQuant(object):  # Interface
    BatchFileInfo: (
        Agilent.MassSpectrometry.DataAnalysis.Quantitative.DynamicQuant.BatchFileInfo
    )  # readonly

    def GetDynamicTargetsInRange(
        self, compoundID: int, lowLevelConc: float, highLevelConc: float
    ) -> List[
        Agilent.MassSpectrometry.DataAnalysis.Quantitative.DynamicQuant.IDynamicQuantTarget
    ]: ...
    def IsTargetCompoundSaturated(self, compoundID: int) -> bool: ...
    def GetDynamicTargetIonGroups(
        self, compoundID: int
    ) -> List[
        Agilent.MassSpectrometry.DataAnalysis.Quantitative.DynamicQuant.ICrossSampleIonGroup
    ]: ...
    def FindCrossSampleIonGroups(self) -> None: ...
    def GetDynamicTargets(
        self, compoundID: int
    ) -> List[
        Agilent.MassSpectrometry.DataAnalysis.Quantitative.DynamicQuant.IDynamicQuantTarget
    ]: ...

class IDynamicQuantTarget(object):  # Interface
    AreaResponseRange: Agilent.MassSpectrometry.DataAnalysis.DoubleRangeD  # readonly
    CalibratedConcentrationRange: (
        Agilent.MassSpectrometry.DataAnalysis.DoubleRangeD
    )  # readonly
    IsUserDefinedTarget: bool  # readonly
    QualifierCount: int  # readonly
    Qualifiers: List[
        Agilent.MassSpectrometry.DataAnalysis.Quantitative.DynamicQuant.IDynamicTargetIon
    ]  # readonly
    Quantifier: (
        Agilent.MassSpectrometry.DataAnalysis.Quantitative.DynamicQuant.IDynamicTargetIon
    )  # readonly

    def GetQualifierByIndex(
        self, qualIndex: int
    ) -> (
        Agilent.MassSpectrometry.DataAnalysis.Quantitative.DynamicQuant.IDynamicTargetIon
    ): ...
    def ContainsQualifier(
        self,
        qualifierIonGroup: Agilent.MassSpectrometry.DataAnalysis.Quantitative.DynamicQuant.IDynamicTargetIon,
    ) -> bool: ...
    def Equals(
        self,
        otherTarget: Agilent.MassSpectrometry.DataAnalysis.Quantitative.DynamicQuant.IDynamicQuantTarget,
    ) -> bool: ...

class IDynamicTargetIon(object):  # Interface
    AreaResponseRange: Agilent.MassSpectrometry.DataAnalysis.DoubleRangeD  # readonly
    CrossSampleIonGroup: (
        Agilent.MassSpectrometry.DataAnalysis.Quantitative.DynamicQuant.ICrossSampleIonGroup
    )  # readonly
    DynamicRange: IntRange  # readonly
    ID: int  # readonly
    MZ: float  # readonly
    SampleAreaResponses: List[float]  # readonly
    SampleHeightResponses: List[float]  # readonly

    def ContainsUnsaturatedIonInSample(self, sampleIndex: int) -> bool: ...

class IIonGroupFeature(object):  # Interface
    FeatureInfo: (
        Agilent.MassSpectrometry.DataAnalysis.Quantitative.DynamicQuant.FeatureInfo
    )  # readonly
    IsDerivedFromComponent: bool  # readonly
    IsUserDefinedTargetIon: bool  # readonly
    Sample: (
        Agilent.MassSpectrometry.DataAnalysis.Quantitative.DynamicQuant.BatchSampleInfo
    )  # readonly
    TargetIon: (
        Agilent.MassSpectrometry.DataAnalysis.Quantitative.DynamicQuant.ITargetIon
    )  # readonly

class IRefIon(object):  # Interface
    Adduct: Agilent.MassSpectrometry.DataAnalysis.MFS.IAdductIon  # readonly
    IsLossOfWater: bool  # readonly
    IsotopeComposition: (
        Agilent.MassSpectrometry.DataAnalysis.MFS.IsotopeComposition
    )  # readonly
    Mass: float  # readonly
    RelativeAbundance: float  # readonly

class ITargetIon(object):  # Interface
    Index: int  # readonly
    MZ: float  # readonly
    MzRange: DoubleRange  # readonly
    SelectedMZ: float  # readonly
    TargetCompound: (
        Agilent.MassSpectrometry.DataAnalysis.Quantitative.DynamicQuant.TargetCompoundInfo
    )  # readonly

class LinearRangeExtensionType(
    System.IConvertible, System.IComparable, System.IFormattable
):  # Struct
    All: (
        Agilent.MassSpectrometry.DataAnalysis.Quantitative.DynamicQuant.LinearRangeExtensionType
    ) = ...  # static # readonly
    UserDefinedTargetIonsOnly: (
        Agilent.MassSpectrometry.DataAnalysis.Quantitative.DynamicQuant.LinearRangeExtensionType
    ) = ...  # static # readonly

class QuantifierMetricType(
    System.IConvertible, System.IComparable, System.IFormattable
):  # Struct
    Heuristic: (
        Agilent.MassSpectrometry.DataAnalysis.Quantitative.DynamicQuant.QuantifierMetricType
    ) = ...  # static # readonly
    MaxAreaResponse: (
        Agilent.MassSpectrometry.DataAnalysis.Quantitative.DynamicQuant.QuantifierMetricType
    ) = ...  # static # readonly
    MaxDynamicRange: (
        Agilent.MassSpectrometry.DataAnalysis.Quantitative.DynamicQuant.QuantifierMetricType
    ) = ...  # static # readonly
    MinConcentrationRSD: (
        Agilent.MassSpectrometry.DataAnalysis.Quantitative.DynamicQuant.QuantifierMetricType
    ) = ...  # static # readonly

class TargetCompoundInfo:  # Class
    CASNumber: str  # readonly
    CompoundID: int  # readonly
    CompoundName: str  # readonly
    Formula: str  # readonly
    Index: int  # readonly
    LeftRTDelta: float  # readonly
    NTargetIons: int  # readonly
    Quantifier: (
        Agilent.MassSpectrometry.DataAnalysis.Quantitative.DynamicQuant.TargetIonInfo
    )  # readonly
    RTRange: DoubleRange  # readonly
    RetentionTime: float  # readonly
    RightRtDelta: float  # readonly
    TargetIons: List[
        Agilent.MassSpectrometry.DataAnalysis.Quantitative.DynamicQuant.TargetIonInfo
    ]  # readonly

    def IsTargetMatch(
        self, mz: float, mzTolerancePpm: float, targetMzDelta: float
    ) -> bool: ...
    def GetTargetIonByIndex(
        self, ionIndex: int
    ) -> (
        Agilent.MassSpectrometry.DataAnalysis.Quantitative.DynamicQuant.TargetIonInfo
    ): ...
    def Equals(
        self,
        other: Agilent.MassSpectrometry.DataAnalysis.Quantitative.DynamicQuant.TargetCompoundInfo,
    ) -> bool: ...

class TargetIonInfo(
    Agilent.MassSpectrometry.DataAnalysis.Quantitative.DynamicQuant.ITargetIon
):  # Class
    Index: int  # readonly
    MZ: float  # readonly
    MzRange: DoubleRange  # readonly
    SelectedMZ: float  # readonly
    TargetCompound: (
        Agilent.MassSpectrometry.DataAnalysis.Quantitative.DynamicQuant.TargetCompoundInfo
    )  # readonly

    def Equals(
        self,
        other: Agilent.MassSpectrometry.DataAnalysis.Quantitative.DynamicQuant.TargetIonInfo,
    ) -> bool: ...



# -*- coding: utf-8 -*-
import typing

# Import specific members from typing used in hints
from typing import (
    Any,
    Callable,
    Dict,
    FrozenSet,
    Generic,
    Iterable,
    Iterator,
    List,
    Mapping,
    Optional,
    Sequence,
    Set,
    Tuple,
    TypeVar,
    Union,
    overload,
)

import datetime
from enum import Enum

from mh_operator.SDK import Agilent, System

from .Utils import CultureItem
from .ViewModel import (
    IControlPanelViewModel,
    ISampleTypesViewModel,
    ITask,
    TaskViewModelBase,
)

# Stubs for namespace: Agilent.MassHunter.Quantitative.Startup.Edit.SampleType

class SampleTypeItem(
    Agilent.MassSpectrometry.DataAnalysis.Quantitative.UIUtils2.NotifyPropertyBase,
    System.ComponentModel.INotifyPropertyChanged,
):  # Class
    def __init__(
        self,
        model: Agilent.MassHunter.Quantitative.Startup.Edit.SampleType.SampleTypesViewModel,
        sampleType: Agilent.MassSpectrometry.DataAnalysis.Quantitative.SampleType,
        visible: bool,
    ) -> None: ...

    Caption: str
    CaptionJa: str
    CaptionRu: str
    CaptionZhHans: str
    Initializing: bool
    IsCaptionDefault: bool  # readonly
    IsCaptionJaDefault: bool  # readonly
    IsCaptionRuDefault: bool  # readonly
    IsCaptionZhHansDefault: bool  # readonly
    IsVisible: bool
    SampleType: (
        Agilent.MassSpectrometry.DataAnalysis.Quantitative.SampleType
    )  # readonly

    def GetCaption(
        self, culture: System.Globalization.CultureInfo, caption: str
    ) -> str: ...

class SampleTypesView(
    System.Windows.Media.Composition.DUCE.IResource,
    System.Windows.Media.Animation.IAnimatable,
    System.Windows.Controls.UserControl,
    System.Windows.IInputElement,
    System.Windows.Markup.IHaveResources,
    System.Windows.Markup.IComponentConnector,
    System.Windows.IFrameworkInputElement,
    System.ComponentModel.ISupportInitialize,
    System.Windows.Markup.IQueryAmbient,
    System.Windows.Markup.IAddChild,
):  # Class
    def __init__(self) -> None: ...
    def InitializeComponent(self) -> None: ...

class SampleTypesViewModel(TaskViewModelBase, ISampleTypesViewModel, ITask):  # Class
    def __init__(self, rootModel: IControlPanelViewModel) -> None: ...

    DefaultColumnVisibilityProperty: System.Windows.DependencyProperty  # static
    JAColumnVisibilityProperty: System.Windows.DependencyProperty  # static
    RUColumnVisibilityProperty: System.Windows.DependencyProperty  # static
    SampleTypeItemsProperty: System.Windows.DependencyProperty  # static
    SelectedLanguageProperty: System.Windows.DependencyProperty  # static
    ZHColumnVisibilityProperty: System.Windows.DependencyProperty  # static

    CommandCheckAll: System.Windows.Input.ICommand  # readonly
    CommandUncheckAll: System.Windows.Input.ICommand  # readonly
    Content: System.Windows.UIElement  # readonly
    Cultures: List[CultureItem]  # readonly
    DefaultColumnVisibility: System.Windows.Visibility
    IsEnabled: bool  # readonly
    JAColumnVisibility: System.Windows.Visibility
    RUColumnVisibility: System.Windows.Visibility
    SampleTypeItems: List[
        Agilent.MassHunter.Quantitative.Startup.Edit.SampleType.SampleTypeItem
    ]  # readonly
    SelectedLanguage: str
    Title: str  # readonly
    View: (
        Agilent.MassHunter.Quantitative.Startup.Edit.SampleType.SampleTypesView
    )  # readonly
    ZHColumnVisibility: System.Windows.Visibility

    def ResetToDefault(self) -> None: ...
    def Reload(self) -> None: ...
    def IsDirty(self) -> bool: ...
    def Activate(self) -> None: ...
    def GetTools(
        self,
    ) -> Agilent.MassHunter.Quantitative.ToolbarWPF.Definitions.Tools: ...

class ToolHandler(
    Agilent.MassSpectrometry.DataAnalysis.Quantitative.Toolbar.IToolHandler
):  # Class
    def __init__(self) -> None: ...
    def Execute(
        self,
        toolState: Agilent.MassSpectrometry.DataAnalysis.Quantitative.Toolbar.IToolState,
        uiState: Any,
    ) -> None: ...
    def SetState(
        self,
        toolState: Agilent.MassSpectrometry.DataAnalysis.Quantitative.Toolbar.IToolState,
        uiState: Any,
    ) -> None: ...



# -*- coding: utf-8 -*-
import typing

# Import specific members from typing used in hints
from typing import (
    Any,
    Callable,
    Dict,
    FrozenSet,
    Generic,
    Iterable,
    Iterator,
    List,
    Mapping,
    Optional,
    Sequence,
    Set,
    Tuple,
    TypeVar,
    Union,
    overload,
)

import datetime
from enum import Enum

from mh_operator.SDK import Agilent, System

# Stubs for namespace: Agilent.MSAcquisition.HashSignature.Properties

class Resources:  # Class
    IDS_ERR_ChecksumDataNotFound: str  # static # readonly



# -*- coding: utf-8 -*-
import typing

# Import specific members from typing used in hints
from typing import (
    Any,
    Callable,
    Dict,
    FrozenSet,
    Generic,
    Iterable,
    Iterator,
    List,
    Mapping,
    Optional,
    Sequence,
    Set,
    Tuple,
    TypeVar,
    Union,
    overload,
)

import datetime
from enum import Enum

from mh_operator.SDK import Agilent, System

# Discovered Generic TypeVars:
T = TypeVar("T")
TLimit = TypeVar("TLimit")
Tlimit = TypeVar("Tlimit")
from . import CompressionScheme, IImsFrameMethod, IImsFrameRecord, ImsFrameScanRec

# Stubs for namespace: Agilent.MassSpectrometry.DataAnalysis.IMS

class FAResourceKeys:  # Class
    BAD_FILE_TYPE: str  # static
    BAD_FILE_VERSION: str  # static
    CANT_LOCK_FILE: str  # static
    CANT_OPEN_FILE: str  # static
    CANT_OPEN_FRAMEBIN: str  # static
    DRIFT_BIN_OUT_OF_RANGE: str  # static
    FILE_NOT_OPEN: str  # static
    FRAME_NUM_OUT_OF_RANGE: str  # static
    FT_BIN_OUT_OF_RANGE: str  # static
    ILLEGAL_ID_VALUE: str  # static
    INVALID_SCHEMA_FILE_VERSION: str  # static
    NONEXISTENT_BINRANGE: str  # static
    NONEXISTENT_FILE: str  # static
    NONEXISTENT_TOFCAL: str  # static
    NONUNIQUE_ID: str  # static
    NOT_ENOUGH_FORMATS: str  # static
    NOT_IMPLEMENTED: str  # static
    OUTPUT_FILE_NOT_OPEN: str  # static
    RT_NONINCREASING: str  # static
    RT_NONUNIQUE: str  # static
    SCHEMA_VALIDATION_ERROR: str  # static
    UNSUPPORTED_FILE_EXTENSION: str  # static
    UNSUPPORTED_VALUE: str  # static

class FrameData:  # Class
    @overload
    def __init__(self) -> None: ...
    @overload
    def __init__(self, autoAccumulateTfs: bool) -> None: ...
    @overload
    def __init__(
        self, src: Agilent.MassSpectrometry.DataAnalysis.IMS.FrameData
    ) -> None: ...

    CentroidTfsSpecArrays: System.Collections.Generic.List[System.Array]  # readonly
    FrameBinRecord: IImsFrameRecord  # readonly
    FrameId: int
    FrameMethod: IImsFrameMethod
    FrameScanCount: int  # readonly
    FrameSpecScanRecList: System.Collections.Generic.List[ImsFrameScanRec]  # readonly
    FrameSpecYArrayCompression: CompressionScheme  # readonly
    FrameSpecYArrayList: System.Collections.Generic.List[List[int]]  # readonly
    FrameSpectrumCount: int  # readonly
    IsFinalizedForStorage: bool  # readonly
    ProfileTfsSpecFirstBin: int  # readonly
    ProfileTfsSpecYArray: List[int]  # readonly
    TfsScanRecord: ImsFrameScanRec  # readonly
    TfsSpecYArrayCompression: CompressionScheme  # readonly
    XStartDeltaArray: List[float]  # readonly

    def ValidateForStorage(self, validationMsg: str) -> bool: ...
    def AddTfsProfileScanRecord(
        self,
        FirstBin: int,
        basePeakAbund: float,
        basePeakMz: float,
        imsScanRec: ImsFrameScanRec,
    ) -> None: ...
    @overload
    def FinalizeForStorage(
        self, nextScanId: int, defaultCal: Agilent.MassSpectrometry.ITofCal
    ) -> None: ...
    @overload
    def FinalizeForStorage(
        self,
        makeCentroidTfs: bool,
        nextScanId: int,
        defaultCal: Agilent.MassSpectrometry.ITofCal,
    ) -> None: ...
    def FrameSpecScanRec(self, specIdx: int) -> ImsFrameScanRec: ...
    def FrameSpecYArray(self, specIdx: int) -> List[int]: ...
    @overload
    def AddSpectrum(self, driftBin: int, abundArray: List[int]) -> None: ...
    @overload
    def AddSpectrum(
        self,
        driftBin: int,
        metrics: Agilent.MassSpectrometry.RlzArrayMetrics,
        encodedArray: List[int],
    ) -> None: ...
    @overload
    def AddTfsProfileSpectrum(self, startNs: float, abundArray: List[int]) -> None: ...
    @overload
    def AddTfsProfileSpectrum(
        self,
        basePeakAbund: float,
        basePeakMz: float,
        metrics: Agilent.MassSpectrometry.RlzArrayMetrics,
        encodedArray: List[int],
    ) -> None: ...
    @overload
    def AddTfsCentroidSpectrum(
        self, nsArray: List[float], abundArray: List[float]
    ) -> None: ...
    @overload
    def AddTfsCentroidSpectrum(
        self, nsArray: List[float], abundArray: List[int]
    ) -> None: ...
    def AddSpectrumScanRecord(
        self, driftBin: int, imsScanRec: ImsFrameScanRec
    ) -> None: ...

class IImsMethod(
    Agilent.MassSpectrometry.DataAnalysis.IMS.IMethParameterSet,
    Agilent.MassSpectrometry.DataAnalysis.IMS.IMethParameter,
    Agilent.MassSpectrometry.DataAnalysis.IMS.IMethParameterXmlIo,
    Iterable[Any],
):  # Interface
    IsFactoryMethod: bool

    def ReadValues(self, sourcePath: str) -> bool: ...
    def SaveValues(self, destinationPath: str) -> None: ...
    def Clone(self) -> Agilent.MassSpectrometry.DataAnalysis.IMS.IImsMethod: ...

class IMethParameter(
    Agilent.MassSpectrometry.DataAnalysis.IMS.IMethParameterXmlIo
):  # Interface
    HasAllowedValues: bool  # readonly
    HasLimits: bool  # readonly
    IsValid: bool  # readonly
    Units: Agilent.MassSpectrometry.MIDAC.MidacUnits  # readonly
    UsageKey: str
    ValidationMessage: str  # readonly
    Value: Any
    ValueString: str  # readonly

    def Equals(
        self, source: Agilent.MassSpectrometry.DataAnalysis.IMS.IMethParameter
    ) -> bool: ...
    def Validate(self) -> bool: ...
    def EqualsValue(
        self, source: Agilent.MassSpectrometry.DataAnalysis.IMS.IMethParameter
    ) -> bool: ...
    def SetValidation(self, validationState: bool, validationMessage: str) -> None: ...
    def EqualsStructure(
        self, source: Agilent.MassSpectrometry.DataAnalysis.IMS.IMethParameter
    ) -> bool: ...
    def Clone(self) -> Agilent.MassSpectrometry.DataAnalysis.IMS.IMethParameter: ...

    ValidationSetEvent: (
        Agilent.MassSpectrometry.DataAnalysis.IMS.ImsValidationSetEventHandler
    )  # Event
    ValueChangedEvent: (
        Agilent.MassSpectrometry.DataAnalysis.IMS.ImsValueChangeEventHandler
    )  # Event

class IMethParameterLimited(
    Agilent.MassSpectrometry.DataAnalysis.IMS.IMethParameter[T],
    Agilent.MassSpectrometry.DataAnalysis.IMS.IMethParameter,
    Agilent.MassSpectrometry.DataAnalysis.IMS.IMethParameterXmlIo,
):  # Interface
    Limits: Agilent.MassSpectrometry.DataAnalysis.IMS.IMethParameterLimits[Tlimit]

    def LimitString(self, withType: bool) -> str: ...

class IMethParameterLimits(object):  # Interface
    Maximum: T
    MaximumLimitType: Agilent.MassSpectrometry.DataAnalysis.IMS.MethParameterLimitType
    Minimum: T
    MinimumLimitType: Agilent.MassSpectrometry.DataAnalysis.IMS.MethParameterLimitType

    def Equals(self, other: Any) -> bool: ...
    def LimitString(self, units: Agilent.MassSpectrometry.MIDAC.MidacUnits) -> str: ...
    def Clone(
        self,
    ) -> Agilent.MassSpectrometry.DataAnalysis.IMS.IMethParameterLimits: ...

class IMethParameterRange(
    Agilent.MassSpectrometry.DataAnalysis.IMS.IMethParameterLimited[
        Agilent.MassSpectrometry.MIDAC.IRange[T], T
    ],
    Agilent.MassSpectrometry.DataAnalysis.IMS.IMethParameter[
        Agilent.MassSpectrometry.MIDAC.IRange[T]
    ],
    Agilent.MassSpectrometry.DataAnalysis.IMS.IMethParameter,
    Agilent.MassSpectrometry.DataAnalysis.IMS.IMethParameterXmlIo,
):  # Interface
    ...

class IMethParameterSet(
    Agilent.MassSpectrometry.DataAnalysis.IMS.IMethParameter,
    Agilent.MassSpectrometry.DataAnalysis.IMS.IMethParameterXmlIo,
    Iterable[Any],
):  # Interface
    Count: int  # readonly
    def __getitem__(
        self, parameterUsageKey: str
    ) -> Agilent.MassSpectrometry.DataAnalysis.IMS.IMethParameter: ...
    UsageKeys: System.Collections.Generic.List[str]  # readonly

    def CopyValues(
        self, source: Agilent.MassSpectrometry.DataAnalysis.IMS.IMethParameterSet
    ) -> None: ...
    def Clone(self) -> Agilent.MassSpectrometry.DataAnalysis.IMS.IMethParameterSet: ...

class IMethParameterXmlIo(object):  # Interface
    def ReadParameter(self, nodeIter: System.Xml.XPath.XPathNodeIterator) -> None: ...
    def WriteParameter(self, writer: System.Xml.XmlWriter) -> None: ...

class IMethParameter(
    Agilent.MassSpectrometry.DataAnalysis.IMS.IMethParameter,
    Agilent.MassSpectrometry.DataAnalysis.IMS.IMethParameterXmlIo,
):  # Interface
    AllowedValues: System.Collections.Generic.List[T]
    Value: T

    def Equals(
        self, source: Agilent.MassSpectrometry.DataAnalysis.IMS.IMethParameter
    ) -> bool: ...
    def Clone(self) -> Agilent.MassSpectrometry.DataAnalysis.IMS.IMethParameter: ...

class ImsFrameMethConstants:  # Class
    DefMassCalId: str = ...  # static # readonly
    DetectorGainDeltaTo: str = ...  # static # readonly
    DetectorGainMode: str = ...  # static # readonly
    DetectorGainRatio: str = ...  # static # readonly
    FileVersion: str = ...  # static # readonly
    FragEnergy: str = ...  # static # readonly
    FragEnergyMode: str = ...  # static # readonly
    FragEnergySegmentDriftBin: str = ...  # static # readonly
    FragEnergySegmentFragEnergy: str = ...  # static # readonly
    FragEnergySegmentPoint: str = ...  # static # readonly
    FragEnergySegments: str = ...  # static # readonly
    FragOpMode: str = ...  # static # readonly
    FrameDtPeriod: str = ...  # static # readonly
    FrameMethId: str = ...  # static # readonly
    FrameMethod: str = ...  # static # readonly
    FrameMethodGroup: str = ...  # static # readonly
    FrameMsXPeriod: str = ...  # static # readonly
    FrameSpecAbundLimit: str = ...  # static # readonly
    FrameSpecFmtId: str = ...  # static # readonly
    FrameType: str = ...  # static # readonly
    ImsField: str = ...  # static # readonly
    ImsGas: str = ...  # static # readonly
    ImsGateMode: str = ...  # static # readonly
    ImsGateOpenTime: str = ...  # static # readonly
    ImsMuxProcessing: str = ...  # static # readonly
    ImsMuxSequence: str = ...  # static # readonly
    ImsTrapMode: str = ...  # static # readonly
    ImsTrapTime: str = ...  # static # readonly
    IonPolarity: str = ...  # static # readonly
    IonizationMode: str = ...  # static # readonly
    MaxMsBin: str = ...  # static # readonly
    MaxMsPerFrame: str = ...  # static # readonly
    MinMsBin: str = ...  # static # readonly
    NumActuals: str = ...  # static # readonly
    NumTransients: str = ...  # static # readonly
    TfsPeakFmtId: str = ...  # static # readonly
    TfsPeakFmtId_Pre: str = ...  # static # readonly
    TfsProfileFmtId: str = ...  # static # readonly
    TfsProfileFmtId_Pre: str = ...  # static # readonly
    TfsStorageMode: str = ...  # static # readonly
    TfsStorageMode_Pre: str = ...  # static # readonly
    Threshold: str = ...  # static # readonly

class ImsFrameSchemaConstants:  # Class
    ActualsOffset: str = ...  # static # readonly
    CycleNumber: str = ...  # static # readonly
    FirstNonzeroDriftBin: str = ...  # static # readonly
    FragClass: str = ...  # static # readonly
    FragEnergy: str = ...  # static # readonly
    FrameBaseAbund: str = ...  # static # readonly
    FrameBaseDriftBin: str = ...  # static # readonly
    FrameBaseMsBin: str = ...  # static # readonly
    FrameId: str = ...  # static # readonly
    FrameMethodId: str = ...  # static # readonly
    FrameRecordType: str = ...  # static # readonly
    FrameScanTime: str = ...  # static # readonly
    FrameSpecAbundLimit: str = ...  # static # readonly
    FrameTic: str = ...  # static # readonly
    ImsField: str = ...  # static # readonly
    ImsPressure: str = ...  # static # readonly
    ImsTemperature: str = ...  # static # readonly
    ImsTrapTime: str = ...  # static # readonly
    IsolationEndMz: str = ...  # static # readonly
    IsolationMz: str = ...  # static # readonly
    IsolationStartMz: str = ...  # static # readonly
    LastNonzeroDriftBin: str = ...  # static # readonly
    MassCalOffset: str = ...  # static # readonly
    NumTransients: str = ...  # static # readonly
    TimeSegmentId: str = ...  # static # readonly

class ImsMethParameterChangeEventArgs(System.EventArgs):  # Class
    def __init__(self, usageKey: str) -> None: ...

    UsageKey: str  # readonly

class ImsMethodParameterSet(
    Agilent.MassSpectrometry.DataAnalysis.IMS.IMethParameterXmlIo,
    Agilent.MassSpectrometry.DataAnalysis.IMS.IMethParameter,
    Iterable[Any],
    Agilent.MassSpectrometry.DataAnalysis.IMS.IImsMethod,
    Agilent.MassSpectrometry.DataAnalysis.IMS.MethParameterSet,
    Agilent.MassSpectrometry.DataAnalysis.IMS.IMethParameterSet,
):  # Class
    @overload
    def __init__(self) -> None: ...
    @overload
    def __init__(
        self, src: Agilent.MassSpectrometry.DataAnalysis.IMS.ImsMethodParameterSet
    ) -> None: ...

    MethodVersion: str

class ImsValidationSetEventHandler(
    System.MulticastDelegate,
    System.ICloneable,
    System.Runtime.Serialization.ISerializable,
):  # Class
    def __init__(self, object: Any, method: System.IntPtr) -> None: ...
    def EndInvoke(self, result: System.IAsyncResult) -> None: ...
    def BeginInvoke(
        self,
        sender: Any,
        args: Agilent.MassSpectrometry.DataAnalysis.IMS.ImsMethParameterChangeEventArgs,
        callback: System.AsyncCallback,
        object: Any,
    ) -> System.IAsyncResult: ...
    def Invoke(
        self,
        sender: Any,
        args: Agilent.MassSpectrometry.DataAnalysis.IMS.ImsMethParameterChangeEventArgs,
    ) -> None: ...

class ImsValueChangeEventHandler(
    System.MulticastDelegate,
    System.ICloneable,
    System.Runtime.Serialization.ISerializable,
):  # Class
    def __init__(self, object: Any, method: System.IntPtr) -> None: ...
    def EndInvoke(self, result: System.IAsyncResult) -> None: ...
    def BeginInvoke(
        self,
        sender: Any,
        args: Agilent.MassSpectrometry.DataAnalysis.IMS.ImsMethParameterChangeEventArgs,
        callback: System.AsyncCallback,
        object: Any,
    ) -> System.IAsyncResult: ...
    def Invoke(
        self,
        sender: Any,
        args: Agilent.MassSpectrometry.DataAnalysis.IMS.ImsMethParameterChangeEventArgs,
    ) -> None: ...

class MethParameterLimitType(
    System.IConvertible, System.IComparable, System.IFormattable
):  # Struct
    Exclusive: Agilent.MassSpectrometry.DataAnalysis.IMS.MethParameterLimitType = (
        ...
    )  # static # readonly
    Inclusive: Agilent.MassSpectrometry.DataAnalysis.IMS.MethParameterLimitType = (
        ...
    )  # static # readonly
    NoLimit: Agilent.MassSpectrometry.DataAnalysis.IMS.MethParameterLimitType = (
        ...
    )  # static # readonly

class MethParameterLimited(
    Agilent.MassSpectrometry.DataAnalysis.IMS.IMethParameter,
    Generic[T, TLimit],
    Agilent.MassSpectrometry.DataAnalysis.IMS.IMethParameterXmlIo,
    Agilent.MassSpectrometry.DataAnalysis.IMS.MethParameter[T],
    Agilent.MassSpectrometry.DataAnalysis.IMS.IMethParameterLimited[T, TLimit],
    Agilent.MassSpectrometry.DataAnalysis.IMS.IMethParameter[T],
):  # Class
    @overload
    def __init__(self) -> None: ...
    @overload
    def __init__(self, units: Agilent.MassSpectrometry.MIDAC.MidacUnits) -> None: ...
    @overload
    def __init__(
        self, src: Agilent.MassSpectrometry.DataAnalysis.IMS.MethParameterLimited
    ) -> None: ...

class MethParameterLimits(
    Generic[T], Agilent.MassSpectrometry.DataAnalysis.IMS.IMethParameterLimits[T]
):  # Class
    @overload
    def __init__(self) -> None: ...
    @overload
    def __init__(
        self, src: Agilent.MassSpectrometry.DataAnalysis.IMS.MethParameterLimits
    ) -> None: ...

class MethParameterMinMaxRange(
    Agilent.MassSpectrometry.DataAnalysis.IMS.IMethParameter[
        Agilent.MassSpectrometry.MIDAC.IRange[T]
    ],
    Generic[T],
    Agilent.MassSpectrometry.DataAnalysis.IMS.IMethParameter,
    Agilent.MassSpectrometry.DataAnalysis.IMS.IMethParameterXmlIo,
    Agilent.MassSpectrometry.DataAnalysis.IMS.MethParameterLimited[
        Agilent.MassSpectrometry.MIDAC.IRange[T], T
    ],
    Agilent.MassSpectrometry.DataAnalysis.IMS.IMethParameterLimited[
        Agilent.MassSpectrometry.MIDAC.IRange[T], T
    ],
    Agilent.MassSpectrometry.DataAnalysis.IMS.IMethParameterRange[T],
):  # Class
    @overload
    def __init__(self) -> None: ...
    @overload
    def __init__(self, units: Agilent.MassSpectrometry.MIDAC.MidacUnits) -> None: ...
    @overload
    def __init__(
        self, src: Agilent.MassSpectrometry.DataAnalysis.IMS.MethParameterMinMaxRange
    ) -> None: ...
    def ToString(self) -> str: ...

class MethParameterSet(
    Agilent.MassSpectrometry.DataAnalysis.IMS.IMethParameterXmlIo,
    Iterable[Any],
    Agilent.MassSpectrometry.DataAnalysis.IMS.IMethParameterSet,
    Agilent.MassSpectrometry.DataAnalysis.IMS.IMethParameter,
):  # Class
    # Nested Types

    class ParameterSetIterator(Iterator[Any]):  # Class
        def __init__(
            self,
            parameterSet: Agilent.MassSpectrometry.DataAnalysis.IMS.MethParameterSet,
        ) -> None: ...

        Current: Agilent.MassSpectrometry.DataAnalysis.IMS.IMethParameter  # readonly

        def MoveNext(self) -> bool: ...
        def Reset(self) -> None: ...

class MethParameter(
    Generic[T],
    Agilent.MassSpectrometry.DataAnalysis.IMS.IMethParameterXmlIo,
    Agilent.MassSpectrometry.DataAnalysis.IMS.IMethParameter[T],
    Agilent.MassSpectrometry.DataAnalysis.IMS.IMethParameter,
):  # Class
    @overload
    def __init__(self) -> None: ...
    @overload
    def __init__(self, units: Agilent.MassSpectrometry.MIDAC.MidacUnits) -> None: ...
    @overload
    def __init__(
        self, src: Agilent.MassSpectrometry.DataAnalysis.IMS.MethParameter
    ) -> None: ...
    def ToString(self) -> str: ...



# -*- coding: utf-8 -*-
import typing

# Import specific members from typing used in hints
from typing import (
    Any,
    Callable,
    Dict,
    FrozenSet,
    Generic,
    Iterable,
    Iterator,
    List,
    Mapping,
    Optional,
    Sequence,
    Set,
    Tuple,
    TypeVar,
    Union,
    overload,
)

import datetime
from enum import Enum

from mh_operator.SDK import Agilent, System

from .FeatureDetection import Feature
from .Quantitative.IndexedData import ScanConditions, ScanSpace

# Stubs for namespace: Agilent.MassSpectrometry.DataAnalysis.FeatureData

class FeatureDataAccess:  # Class
    def __init__(self) -> None: ...

    IsOpen: bool  # readonly
    ScanConditionList: System.Collections.Generic.List[ScanConditions]  # readonly

    def GetFeatureSet(
        self, scanConditions: ScanConditions
    ) -> Agilent.MassSpectrometry.DataAnalysis.FeatureData.IFeatureSetQuery: ...
    def GetSaturatedFeatureCount(self) -> int: ...
    def Open(self, sampleDataPath: str) -> bool: ...
    def WriteFeaturesToFile(
        self, features: System.Collections.Generic.List[Feature], sampleDataPath: str
    ) -> None: ...
    def GetTotalFeatureCount(self) -> int: ...
    def Close(self) -> None: ...

class FeatureFile(System.IDisposable):  # Class
    def __init__(self) -> None: ...

    IsOpen: bool  # readonly
    IsReadOnly: bool  # readonly

    def Read(
        self,
    ) -> Agilent.MassSpectrometry.DataAnalysis.FeatureData.SampleFeatures: ...
    def Write(
        self, sf: Agilent.MassSpectrometry.DataAnalysis.FeatureData.SampleFeatures
    ) -> None: ...
    def Open(self, sampleDataPath: str, readOnly: bool) -> None: ...
    def Close(self) -> None: ...
    @staticmethod
    def Exists(sampleDataPath: str) -> bool: ...
    def Dispose(self) -> None: ...

class FeatureSet(
    Agilent.MassSpectrometry.DataAnalysis.FeatureData.IFeatureSetQuery
):  # Class
    @overload
    def __init__(self, scanSpace: ScanSpace) -> None: ...
    @overload
    def __init__(
        self, scanSpace: ScanSpace, features: System.Collections.Generic.List[Feature]
    ) -> None: ...

    Count: int  # readonly
    SaturatedCount: int  # readonly
    ScanSpace: ScanSpace  # readonly

    @overload
    def GetFeatures(self) -> System.Collections.Generic.List[Feature]: ...
    @overload
    def GetFeatures(
        self, rtStart: float, rtEnd: float, mzLow: float, mzHigh: float
    ) -> System.Collections.Generic.List[Feature]: ...
    def GetCoelutingFeatures(
        self, f: Feature
    ) -> System.Collections.Generic.List[Feature]: ...
    def ReadFeatures(self, br: System.IO.BinaryReader) -> bool: ...
    def GetFeaturesInRTRange(
        self, rtStart: float, rtEnd: float
    ) -> System.Collections.Generic.List[Feature]: ...
    def WriteFeatures(self, bw: System.IO.BinaryWriter) -> None: ...
    def GetFeaturesInMzRange(
        self, mzLow: float, mzHigh: float
    ) -> System.Collections.Generic.List[Feature]: ...

class IFeatureSetQuery(object):  # Interface
    Count: int  # readonly
    SaturatedCount: int  # readonly
    ScanSpace: ScanSpace  # readonly

    def GetCoelutingFeatures(
        self, f: Feature
    ) -> System.Collections.Generic.List[Feature]: ...
    @overload
    def GetFeatures(self) -> System.Collections.Generic.List[Feature]: ...
    @overload
    def GetFeatures(
        self, rtStart: float, rtEnd: float, mzLow: float, mzHigh: float
    ) -> System.Collections.Generic.List[Feature]: ...
    def GetFeaturesInRTRange(
        self, rtStart: float, rtEnd: float
    ) -> System.Collections.Generic.List[Feature]: ...
    def GetFeaturesInMzRange(
        self, mzLow: float, mzHigh: float
    ) -> System.Collections.Generic.List[Feature]: ...

class SampleFeatures:  # Class
    @overload
    def __init__(self) -> None: ...
    @overload
    def __init__(self, features: System.Collections.Generic.List[Feature]) -> None: ...

    ScanConditionCount: int  # readonly
    ScanConditionList: System.Collections.Generic.List[ScanConditions]  # readonly

    def GetFeatureSet(
        self, scanConditions: ScanConditions
    ) -> Agilent.MassSpectrometry.DataAnalysis.FeatureData.FeatureSet: ...
    def GetScanSpace(self, scanConditions: ScanConditions) -> ScanSpace: ...
    def AddFeatureSet(
        self,
        scanSpace: ScanSpace,
        featureSet: Agilent.MassSpectrometry.DataAnalysis.FeatureData.FeatureSet,
    ) -> None: ...



# -*- coding: utf-8 -*-
import typing

# Import specific members from typing used in hints
from typing import (
    Any,
    Callable,
    Dict,
    FrozenSet,
    Generic,
    Iterable,
    Iterator,
    List,
    Mapping,
    Optional,
    Sequence,
    Set,
    Tuple,
    TypeVar,
    Union,
    overload,
)

import datetime
from enum import Enum

from mh_operator.SDK import Agilent, System

from . import Definitions, ToolManager, ToolState

# Stubs for namespace: Agilent.MassHunter.Quantitative.ToolbarWPF

class ImageConverter(
    System.Windows.DependencyObject, System.Windows.Data.IValueConverter
):  # Class
    @overload
    def __init__(self) -> None: ...
    @overload
    def __init__(self, color: System.Windows.Media.Color) -> None: ...

    ColorProperty: System.Windows.DependencyProperty  # static # readonly

    Color: System.Windows.Media.Color

    def ConvertBack(
        self,
        value_: Any,
        targetType: System.Type,
        parameter: Any,
        culture: System.Globalization.CultureInfo,
    ) -> Any: ...
    def Convert(
        self,
        value_: Any,
        targetType: System.Type,
        parameter: Any,
        culture: System.Globalization.CultureInfo,
    ) -> Any: ...
